@@ -1,57 +1,49 @@
-use vortex_array::array::{PrimitiveArray, TemporalArray};
+use vortex_array::arrays::{PrimitiveArray, TemporalArray};
 use vortex_array::compute::try_cast;
-use vortex_array::{Array, IntoArray, IntoArrayVariant};
+use vortex_array::{Array, ArrayRef, IntoArray, ToCanonical};
 use vortex_buffer::BufferMut;
-use vortex_datetime_dtype::TimeUnit;
 use vortex_dtype::{DType, PType};
-use vortex_error::{vortex_bail, VortexError, VortexResult};
+use vortex_error::{VortexError, VortexResult};
 
-use crate::DateTimePartsArray;
+use crate::{DateTimePartsArray, timestamp};
 
 pub struct TemporalParts {
-    pub days: Array,
-    pub seconds: Array,
-    pub subseconds: Array,
+    pub days: ArrayRef,
+    pub seconds: ArrayRef,
+    pub subseconds: ArrayRef,
 }
 
-/// Compress a `TemporalArray` into day, second, and subsecond components.
+/// Compress a `TemporalArray` into day, second, and subseconds components.
 ///
 /// Splitting the components by granularity creates more small values, which enables better
 /// cascading compression.
 pub fn split_temporal(array: TemporalArray) -> VortexResult<TemporalParts> {
-    let temporal_values = array.temporal_values().into_primitive()?;
-    let validity = temporal_values.validity();
+    let temporal_values = array.temporal_values().to_primitive()?;
+    let validity = temporal_values.validity().clone();
 
     // After this operation, timestamps will be non-nullable PrimitiveArray<i64>
     let timestamps = try_cast(
         &temporal_values,
         &DType::Primitive(PType::I64, temporal_values.dtype().nullability()),
     )?
-    .into_primitive()?;
-
-    let divisor = match array.temporal_metadata().time_unit() {
-        TimeUnit::Ns => 1_000_000_000,
-        TimeUnit::Us => 1_000_000,
-        TimeUnit::Ms => 1_000,
-        TimeUnit::S => 1,
-        TimeUnit::D => vortex_bail!(InvalidArgument: "Cannot compress day-level data"),
-    };
+    .to_primitive()?;
 
     let length = timestamps.len();
     let mut days = BufferMut::with_capacity(length);
     let mut seconds = BufferMut::with_capacity(length);
-    let mut subsecond = BufferMut::with_capacity(length);
+    let mut subseconds = BufferMut::with_capacity(length);
 
-    for &t in timestamps.as_slice::<i64>().iter() {
-        days.push(t / (86_400 * divisor));
-        seconds.push((t % (86_400 * divisor)) / divisor);
-        subsecond.push((t % (86_400 * divisor)) % divisor);
+    for &ts in timestamps.as_slice::<i64>().iter() {
+        let ts_parts = timestamp::split(ts, array.temporal_metadata().time_unit())?;
+        days.push(ts_parts.days);
+        seconds.push(ts_parts.seconds);
+        subseconds.push(ts_parts.subseconds);
     }
 
     Ok(TemporalParts {
         days: PrimitiveArray::new(days, validity).into_array(),
         seconds: seconds.into_array(),
-        subseconds: subsecond.into_array(),
+        subseconds: subseconds.into_array(),
     })
 }
 
@@ -72,13 +64,13 @@ impl TryFrom<TemporalArray> for DateTimePartsArray {
 #[cfg(test)]
 mod tests {
     use rstest::rstest;
-    use vortex_array::array::{PrimitiveArray, TemporalArray};
+    use vortex_array::arrays::{PrimitiveArray, TemporalArray};
     use vortex_array::validity::Validity;
-    use vortex_array::{IntoArray as _, IntoArrayVariant as _};
+    use vortex_array::{Array, ToCanonical};
     use vortex_buffer::buffer;
     use vortex_datetime_dtype::TimeUnit;
 
-    use crate::{split_temporal, TemporalParts};
+    use crate::{TemporalParts, split_temporal};
 
     #[rstest]
     #[case(Validity::NonNullable)]
@@ -102,14 +94,14 @@ mod tests {
             seconds,
             subseconds,
         } = split_temporal(temporal_array).unwrap();
-        assert_eq!(days.into_primitive().unwrap().validity(), validity);
+        assert_eq!(days.to_primitive().unwrap().validity(), &validity);
         assert_eq!(
-            seconds.into_primitive().unwrap().validity(),
-            Validity::NonNullable
+            seconds.to_primitive().unwrap().validity(),
+            &Validity::NonNullable
         );
         assert_eq!(
-            subseconds.into_primitive().unwrap().validity(),
-            Validity::NonNullable
+            subseconds.to_primitive().unwrap().validity(),
+            &Validity::NonNullable
         );
     }
 }
