@@ -5,4 +5,6 @@ mod array;
 mod canonical;
 mod compress;
 mod compute;
+mod serde;
 mod stats;
+mod timestamp;
