@@ -3,10 +3,12 @@
 //! Expose a [DictArray] which is zero-copy equivalent to Arrow's
 //! [DictionaryArray](https://docs.rs/arrow/latest/arrow/array/struct.DictionaryArray.html).
 pub use array::*;
-pub use compress::*;
 
 mod array;
-mod compress;
+pub mod builders;
 mod compute;
+mod serde;
 mod stats;
+#[cfg(feature = "test-harness")]
+pub mod test;
 mod variants;
