@@ -1,45 +1,52 @@
-use std::fmt::{Debug, Display};
+use std::fmt::Debug;
 
 pub use compress::*;
 use fastlanes::BitPacking;
-use vortex_array::array::PrimitiveArray;
-use vortex_array::patches::{Patches, PatchesMetadata};
-use vortex_array::stats::StatsSet;
-use vortex_array::validity::{Validity, ValidityMetadata};
+use vortex_array::arrays::PrimitiveArray;
+use vortex_array::builders::ArrayBuilder;
+use vortex_array::patches::Patches;
+use vortex_array::stats::{ArrayStats, StatsSetRef};
+use vortex_array::validity::Validity;
 use vortex_array::variants::PrimitiveArrayTrait;
-use vortex_array::visitor::ArrayVisitor;
-use vortex_array::vtable::{
-    CanonicalVTable, StatisticsVTable, ValidateVTable, ValidityVTable, VariantsVTable,
-    VisitorVTable,
+use vortex_array::vtable::{EncodingVTable, StatisticsVTable, VTableRef};
+use vortex_array::{
+    Array, ArrayCanonicalImpl, ArrayExt, ArrayImpl, ArrayStatisticsImpl, ArrayValidityImpl,
+    ArrayVariantsImpl, Canonical, Encoding, EncodingId, RkyvMetadata, try_from_array_ref,
 };
-use vortex_array::{encoding_ids, impl_encoding, Array, Canonical, RkyvMetadata};
 use vortex_buffer::ByteBuffer;
-use vortex_dtype::{DType, NativePType, PType};
-use vortex_error::{vortex_bail, vortex_err, VortexExpect as _, VortexResult};
+use vortex_dtype::{DType, NativePType, PType, match_each_integer_ptype_with_unsigned_type};
+use vortex_error::{VortexExpect as _, VortexResult, vortex_bail, vortex_err};
 use vortex_mask::Mask;
 
+use crate::bitpacking::serde::BitPackedMetadata;
+
 mod compress;
 mod compute;
+mod serde;
 
-impl_encoding!(
-    "fastlanes.bitpacked",
-    encoding_ids::FL_BITPACKED,
-    BitPacked,
-    RkyvMetadata<BitPackedMetadata>
-);
-
-#[derive(Debug, Clone, rkyv::Archive, rkyv::Serialize, rkyv::Deserialize)]
-#[repr(C)]
-pub struct BitPackedMetadata {
-    validity: ValidityMetadata,
+#[derive(Clone, Debug)]
+pub struct BitPackedArray {
+    offset: u16,
+    len: usize,
+    dtype: DType,
     bit_width: u8,
-    offset: u16, // must be <1024
-    patches: Option<PatchesMetadata>,
+    packed: ByteBuffer,
+    patches: Option<Patches>,
+    validity: Validity,
+    stats_set: ArrayStats,
 }
 
-impl Display for BitPackedMetadata {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        Debug::fmt(self, f)
+try_from_array_ref!(BitPackedArray);
+
+pub struct BitPackedEncoding;
+impl Encoding for BitPackedEncoding {
+    type Array = BitPackedArray;
+    type Metadata = RkyvMetadata<BitPackedMetadata>;
+}
+
+impl EncodingVTable for BitPackedEncoding {
+    fn id(&self) -> EncodingId {
+        EncodingId::new_ref("fastlanes.bitpacked")
     }
 }
 
@@ -119,7 +126,7 @@ impl BitPackedArray {
 
         // expected packed size is in bytes
         let expected_packed_size =
-            ((length + offset as usize + 1023) / 1024) * (128 * bit_width as usize);
+            (length + offset as usize).div_ceil(1024) * (128 * bit_width as usize);
         if packed.len() != expected_packed_size {
             return Err(vortex_err!(
                 "Expected {} packed bytes, got {}",
@@ -132,40 +139,21 @@ impl BitPackedArray {
         //  enforce custom alignments.
         // let packed = ByteBuffer::new_with_alignment(packed, FASTLANES_ALIGNMENT);
 
-        let metadata = BitPackedMetadata {
-            validity: validity.to_metadata(length)?,
+        Ok(Self {
             offset,
-            bit_width,
-            patches: patches
-                .as_ref()
-                .map(|p| p.to_metadata(length, &dtype))
-                .transpose()?,
-        };
-
-        let mut children = Vec::with_capacity(3);
-        if let Some(p) = patches.as_ref() {
-            children.push(p.indices().clone());
-            children.push(p.values().clone());
-        }
-        if let Some(a) = validity.into_array() {
-            children.push(a)
-        }
-
-        Self::try_from_parts(
+            len: length,
             dtype,
-            length,
-            RkyvMetadata(metadata),
-            Some([packed].into()),
-            Some(children.into()),
-            StatsSet::default(),
-        )
+            bit_width,
+            packed,
+            patches,
+            validity,
+            stats_set: Default::default(),
+        })
     }
 
     #[inline]
     pub fn packed(&self) -> &ByteBuffer {
-        self.as_ref()
-            .byte_buffer(0)
-            .vortex_expect("BitPackedArray must contain packed buffer")
+        &self.packed
     }
 
     /// Access the slice of packed values as an array of `T`
@@ -184,7 +172,7 @@ impl BitPackedArray {
 
     #[inline]
     pub fn bit_width(&self) -> u8 {
-        self.metadata().bit_width
+        self.bit_width
     }
 
     /// Access the patches array.
@@ -192,36 +180,21 @@ impl BitPackedArray {
     /// If present, patches MUST be a `SparseArray` with equal-length to this array, and whose
     /// indices indicate the locations of patches. The indices must have non-zero length.
     #[inline]
-    pub fn patches(&self) -> Option<Patches> {
-        self.metadata().patches.as_ref().map(|patches| {
-            Patches::new(
-                self.len(),
-                self.as_ref()
-                    .child(0, &patches.indices_dtype(), patches.len())
-                    .vortex_expect("BitPackedArray: patch indices"),
-                self.as_ref()
-                    .child(1, self.dtype(), patches.len())
-                    .vortex_expect("BitPackedArray: patch values"),
-            )
-        })
+    pub fn patches(&self) -> Option<&Patches> {
+        self.patches.as_ref()
+    }
+
+    pub fn replace_patches(&mut self, patches: Option<Patches>) {
+        self.patches = patches;
     }
 
     #[inline]
     pub fn offset(&self) -> u16 {
-        self.metadata().offset
+        self.offset
     }
 
-    pub fn validity(&self) -> Validity {
-        let validity_child_idx = if self.metadata().patches.is_some() {
-            2
-        } else {
-            0
-        };
-        self.metadata().validity.to_validity(|| {
-            self.as_ref()
-                .child(validity_child_idx, &Validity::DTYPE, self.len())
-                .vortex_expect("BitPackedArray: validity child")
-        })
+    pub fn validity(&self) -> &Validity {
+        &self.validity
     }
 
     /// Bit-pack an array of primitive integers down to the target bit-width using the FastLanes
@@ -235,8 +208,8 @@ impl BitPackedArray {
     ///
     /// If the requested bit-width for packing is larger than the array's native width, an
     /// error will be returned.
-    pub fn encode(array: &Array, bit_width: u8) -> VortexResult<Self> {
-        if let Ok(parray) = PrimitiveArray::try_from(array.clone()) {
+    pub fn encode(array: &dyn Array, bit_width: u8) -> VortexResult<Self> {
+        if let Some(parray) = array.as_opt::<PrimitiveArray>() {
             bitpack_encode(parray, bit_width)
         } else {
             vortex_bail!("Bitpacking can only encode primitive arrays");
@@ -252,85 +225,109 @@ impl BitPackedArray {
     }
 }
 
-impl CanonicalVTable<BitPackedArray> for BitPackedEncoding {
-    fn into_canonical(&self, array: BitPackedArray) -> VortexResult<Canonical> {
-        unpack(array).map(Canonical::Primitive)
+impl ArrayImpl for BitPackedArray {
+    type Encoding = BitPackedEncoding;
+
+    fn _len(&self) -> usize {
+        self.len
+    }
+
+    fn _dtype(&self) -> &DType {
+        &self.dtype
+    }
+
+    fn _vtable(&self) -> VTableRef {
+        VTableRef::new_ref(&BitPackedEncoding)
     }
 }
 
-impl ValidityVTable<BitPackedArray> for BitPackedEncoding {
-    fn is_valid(&self, array: &BitPackedArray, index: usize) -> VortexResult<bool> {
-        array.validity().is_valid(index)
+impl ArrayCanonicalImpl for BitPackedArray {
+    fn _to_canonical(&self) -> VortexResult<Canonical> {
+        unpack(self).map(Canonical::Primitive)
     }
 
-    fn logical_validity(&self, array: &BitPackedArray) -> VortexResult<Mask> {
-        array.validity().to_logical(array.len())
+    fn _append_to_builder(&self, builder: &mut dyn ArrayBuilder) -> VortexResult<()> {
+        match_each_integer_ptype_with_unsigned_type!(self.ptype(), |$T, $UnsignedT| {
+            unpack_into::<$T, $UnsignedT, _, _>(
+                self,
+                builder
+                    .as_any_mut()
+                    .downcast_mut()
+                    .vortex_expect("bit packed array must canonicalize into a primitive array"),
+                // SAFETY: UnsignedT is the unsigned verison of T, reinterpreting &[UnsignedT] to
+                // &[T] is therefore safe.
+                |x| unsafe { std::mem::transmute(x) },
+                // SAFETY: UnsignedT is the unsigned verison of T, reinterpreting &mut [T] to
+                // &mut [UnsignedT] is therefore safe.
+                |x| unsafe { std::mem::transmute(x) },
+            )
+        })
     }
 }
 
-impl VisitorVTable<BitPackedArray> for BitPackedEncoding {
-    fn accept(&self, array: &BitPackedArray, visitor: &mut dyn ArrayVisitor) -> VortexResult<()> {
-        visitor.visit_buffer(array.packed())?;
-        if let Some(patches) = array.patches().as_ref() {
-            visitor.visit_patches(patches)?;
-        }
-        visitor.visit_validity(&array.validity())
+impl ArrayStatisticsImpl for BitPackedArray {
+    fn _stats_ref(&self) -> StatsSetRef<'_> {
+        self.stats_set.to_ref(self)
     }
 }
 
-impl StatisticsVTable<BitPackedArray> for BitPackedEncoding {}
+impl ArrayValidityImpl for BitPackedArray {
+    fn _is_valid(&self, index: usize) -> VortexResult<bool> {
+        self.validity.is_valid(index)
+    }
 
-impl ValidateVTable<BitPackedArray> for BitPackedEncoding {}
+    fn _all_valid(&self) -> VortexResult<bool> {
+        self.validity.all_valid()
+    }
 
-impl VariantsVTable<BitPackedArray> for BitPackedEncoding {
-    fn as_primitive_array<'a>(
-        &self,
-        array: &'a BitPackedArray,
-    ) -> Option<&'a dyn PrimitiveArrayTrait> {
-        Some(array)
+    fn _all_invalid(&self) -> VortexResult<bool> {
+        self.validity.all_invalid()
+    }
+
+    fn _validity_mask(&self) -> VortexResult<Mask> {
+        self.validity.to_logical(self.len())
     }
 }
 
+impl ArrayVariantsImpl for BitPackedArray {
+    fn _as_primitive_typed(&self) -> Option<&dyn PrimitiveArrayTrait> {
+        Some(self)
+    }
+}
+
+impl StatisticsVTable<&BitPackedArray> for BitPackedEncoding {}
+
 impl PrimitiveArrayTrait for BitPackedArray {}
 
 #[cfg(test)]
 mod test {
-    use vortex_array::array::PrimitiveArray;
-    use vortex_array::patches::PatchesMetadata;
-    use vortex_array::test_harness::check_metadata;
-    use vortex_array::validity::ValidityMetadata;
-    use vortex_array::{IntoArray, IntoArrayVariant, RkyvMetadata};
+    use vortex_array::arrays::PrimitiveArray;
+    use vortex_array::{IntoArray, ToCanonical};
     use vortex_buffer::Buffer;
-    use vortex_dtype::PType;
 
-    use crate::{BitPackedArray, BitPackedMetadata};
+    use crate::BitPackedArray;
 
-    #[cfg_attr(miri, ignore)]
-    #[test]
-    fn test_bitpacked_metadata() {
-        check_metadata(
-            "bitpacked.metadata",
-            RkyvMetadata(BitPackedMetadata {
-                patches: Some(PatchesMetadata::new(usize::MAX, PType::U64)),
-                validity: ValidityMetadata::AllValid,
-                offset: u16::MAX,
-                bit_width: u8::MAX,
-            }),
-        );
-    }
+    // #[cfg_attr(miri, ignore)]
+    // #[test]
+    // fn test_bitpacked_metadata() {
+    //     check_metadata(
+    //         "bitpacked.metadata",
+    //         RkyvMetadata(BitPackedMetadata {
+    //             patches: Some(PatchesMetadata::new(usize::MAX, usize::MAX, PType::U64)),
+    //             validity: ValidityMetadata::AllValid,
+    //             offset: u16::MAX,
+    //             bit_width: u8::MAX,
+    //         }),
+    //     );
+    // }
 
     #[test]
     fn test_encode() {
         let values = [Some(1), None, Some(1), None, Some(1), None, Some(u64::MAX)];
         let uncompressed = PrimitiveArray::from_option_iter(values);
-        let packed = BitPackedArray::encode(uncompressed.as_ref(), 1).unwrap();
+        let packed = BitPackedArray::encode(&uncompressed, 1).unwrap();
         let expected = &[1, 0, 1, 0, 1, 0, u64::MAX];
-        let results = packed
-            .into_array()
-            .into_primitive()
-            .unwrap()
-            .as_slice::<u64>()
-            .to_vec();
+        let results = packed.to_primitive().unwrap().as_slice::<u64>().to_vec();
         assert_eq!(results, expected);
     }
 
@@ -338,9 +335,9 @@ mod test {
     fn test_encode_too_wide() {
         let values = [Some(1u8), None, Some(1), None, Some(1), None];
         let uncompressed = PrimitiveArray::from_option_iter(values);
-        let _packed = BitPackedArray::encode(uncompressed.as_ref(), 8)
+        let _packed = BitPackedArray::encode(&uncompressed, 8)
             .expect_err("Cannot pack value into the same width");
-        let _packed = BitPackedArray::encode(uncompressed.as_ref(), 9)
+        let _packed = BitPackedArray::encode(&uncompressed, 9)
             .expect_err("Cannot pack value into larger width");
     }
 
@@ -353,7 +350,7 @@ mod test {
         assert!(packed_with_patches.patches().is_some());
         assert_eq!(
             packed_with_patches
-                .into_primitive()
+                .to_primitive()
                 .unwrap()
                 .as_slice::<i32>(),
             values.as_slice()
