@@ -1,52 +1,78 @@
 use fsst::Symbol;
-use vortex_array::array::ConstantArray;
-use vortex_array::compute::{compare, CompareFn, Operator};
-use vortex_array::{Array, IntoArray, IntoArrayVariant};
+use vortex_array::arrays::{BoolArray, BooleanBuffer, ConstantArray};
+use vortex_array::compute::{CompareFn, Operator, compare, compare_lengths_to_empty};
+use vortex_array::validity::Validity;
+use vortex_array::variants::PrimitiveArrayTrait;
+use vortex_array::{Array, ArrayRef, ToCanonical};
 use vortex_buffer::ByteBuffer;
-use vortex_dtype::{DType, Nullability};
-use vortex_error::{VortexExpect, VortexResult};
-use vortex_scalar::Scalar;
+use vortex_dtype::{DType, match_each_native_ptype};
+use vortex_error::{VortexExpect, VortexResult, vortex_bail};
 
 use crate::{FSSTArray, FSSTEncoding};
 
-impl CompareFn<FSSTArray> for FSSTEncoding {
+impl CompareFn<&FSSTArray> for FSSTEncoding {
     fn compare(
         &self,
         lhs: &FSSTArray,
-        rhs: &Array,
+        rhs: &dyn Array,
         operator: Operator,
-    ) -> VortexResult<Option<Array>> {
-        match (rhs.as_constant(), operator) {
-            (Some(constant), _) if constant.is_null() => {
-                // All comparisons to null must return null
-                Ok(Some(
-                    ConstantArray::new(Scalar::null(DType::Bool(Nullability::Nullable)), lhs.len())
-                        .into_array(),
-                ))
+    ) -> VortexResult<Option<ArrayRef>> {
+        match rhs.as_constant() {
+            Some(constant) => {
+                compare_fsst_constant(lhs, &ConstantArray::new(constant, lhs.len()), operator)
             }
-            (Some(constant), Operator::Eq | Operator::NotEq) => compare_fsst_constant(
-                lhs,
-                &ConstantArray::new(constant, lhs.len()),
-                operator == Operator::Eq,
-            )
-            .map(Some),
             // Otherwise, fall back to the default comparison behavior.
             _ => Ok(None),
         }
     }
 }
 
-/// Specialized compare function implementation used when performing equals or not equals against
-/// a constant.
+/// Specialized compare function implementation used when performing against a constant
 fn compare_fsst_constant(
     left: &FSSTArray,
     right: &ConstantArray,
-    equal: bool,
-) -> VortexResult<Array> {
-    let symbols = left.symbols().into_primitive()?;
+    operator: Operator,
+) -> VortexResult<Option<ArrayRef>> {
+    let rhs_scalar = right.scalar();
+    let is_rhs_empty = match rhs_scalar.dtype() {
+        DType::Binary(_) => rhs_scalar
+            .as_binary()
+            .is_empty()
+            .vortex_expect("RHS should not be null"),
+        DType::Utf8(_) => rhs_scalar
+            .as_utf8()
+            .is_empty()
+            .vortex_expect("RHS should not be null"),
+        _ => vortex_bail!("VarBinArray can only have type of Binary or Utf8"),
+    };
+    if is_rhs_empty {
+        let buffer = match operator {
+            // Every possible value is gte ""
+            Operator::Gte => BooleanBuffer::new_set(left.len()),
+            // No value is lt ""
+            Operator::Lt => BooleanBuffer::new_unset(left.len()),
+            _ => {
+                let uncompressed_lengths = left.uncompressed_lengths().to_primitive()?;
+                match_each_native_ptype!(uncompressed_lengths.ptype(), |$P| {
+                    compare_lengths_to_empty(uncompressed_lengths.as_slice::<$P>().iter().copied(), operator)
+                })
+            }
+        };
+
+        return Ok(Some(
+            BoolArray::new(buffer, Validity::copy_from_array(left)?).into_array(),
+        ));
+    }
+
+    // The following section only supports Eq/NotEq
+    if !matches!(operator, Operator::Eq | Operator::NotEq) {
+        return Ok(None);
+    }
+
+    let symbols = left.symbols().to_primitive()?;
     let symbols_u64 = symbols.as_slice::<u64>();
 
-    let symbol_lens = left.symbol_lengths().into_primitive()?;
+    let symbol_lens = left.symbol_lengths().to_primitive()?;
     let symbol_lens_u8 = symbol_lens.as_slice::<u8>();
 
     let mut compressor = fsst::CompressorBuilder::new();
@@ -76,18 +102,14 @@ fn compare_fsst_constant(
     };
 
     let rhs = ConstantArray::new(encoded_scalar, left.len());
-    compare(
-        left.codes(),
-        rhs,
-        if equal { Operator::Eq } else { Operator::NotEq },
-    )
+    compare(left.codes(), &rhs, operator).map(Some)
 }
 
 #[cfg(test)]
 mod tests {
-    use vortex_array::array::{ConstantArray, VarBinArray};
-    use vortex_array::compute::{compare, scalar_at, Operator};
-    use vortex_array::{IntoArray, IntoArrayVariant};
+    use vortex_array::arrays::{ConstantArray, VarBinArray};
+    use vortex_array::compute::{Operator, compare, scalar_at};
+    use vortex_array::{Array, ToCanonical};
     use vortex_dtype::{DType, Nullability};
     use vortex_scalar::Scalar;
 
@@ -105,17 +127,16 @@ mod tests {
                 Some("this is a very long string"),
             ],
             DType::Utf8(Nullability::Nullable),
-        )
-        .into_array();
+        );
         let compressor = fsst_train_compressor(&lhs).unwrap();
         let lhs = fsst_compress(&lhs, &compressor).unwrap();
 
-        let rhs = ConstantArray::new("world", lhs.len()).into_array();
+        let rhs = ConstantArray::new("world", lhs.len());
 
         // Ensure fastpath for Eq exists, and returns correct answer
         let equals: Vec<bool> = compare(&lhs, &rhs, Operator::Eq)
             .unwrap()
-            .into_bool()
+            .to_bool()
             .unwrap()
             .boolean_buffer()
             .into_iter()
@@ -126,7 +147,7 @@ mod tests {
         // Ensure fastpath for Eq exists, and returns correct answer
         let not_equals: Vec<bool> = compare(&lhs, &rhs, Operator::NotEq)
             .unwrap()
-            .into_bool()
+            .to_bool()
             .unwrap()
             .boolean_buffer()
             .into_iter()
@@ -137,12 +158,12 @@ mod tests {
         // Ensure null constants are handled correctly.
         let null_rhs =
             ConstantArray::new(Scalar::null(DType::Utf8(Nullability::Nullable)), lhs.len());
-        let equals_null = compare(&lhs, null_rhs.as_ref(), Operator::Eq).unwrap();
+        let equals_null = compare(&lhs, &null_rhs, Operator::Eq).unwrap();
         for idx in 0..lhs.len() {
             assert!(scalar_at(&equals_null, idx).unwrap().is_null());
         }
 
-        let noteq_null = compare(&lhs, null_rhs.as_ref(), Operator::NotEq).unwrap();
+        let noteq_null = compare(&lhs, &null_rhs, Operator::NotEq).unwrap();
         for idx in 0..lhs.len() {
             assert!(scalar_at(&noteq_null, idx).unwrap().is_null());
         }
