@@ -1,40 +1,41 @@
-use vortex_array::array::ConstantArray;
-use vortex_array::compute::{binary_numeric, BinaryNumericFn};
-use vortex_array::{Array, IntoArray};
+use vortex_array::arrays::ConstantArray;
+use vortex_array::compute::{BinaryNumericFn, binary_numeric};
+use vortex_array::{Array, ArrayRef};
 use vortex_error::VortexResult;
 use vortex_scalar::BinaryNumericOperator;
 
 use crate::{RunEndArray, RunEndEncoding};
 
-impl BinaryNumericFn<RunEndArray> for RunEndEncoding {
+impl BinaryNumericFn<&RunEndArray> for RunEndEncoding {
     fn binary_numeric(
         &self,
         array: &RunEndArray,
-        rhs: &Array,
+        rhs: &dyn Array,
         op: BinaryNumericOperator,
-    ) -> VortexResult<Option<Array>> {
+    ) -> VortexResult<Option<ArrayRef>> {
         let Some(rhs_scalar) = rhs.as_constant() else {
             return Ok(None);
         };
 
         let rhs_const_array = ConstantArray::new(rhs_scalar, array.values().len()).into_array();
 
-        RunEndArray::with_offset_and_length(
-            array.ends(),
-            binary_numeric(&array.values(), &rhs_const_array, op)?,
-            array.offset(),
-            array.len(),
-        )
-        .map(IntoArray::into_array)
-        .map(Some)
+        Ok(Some(
+            RunEndArray::with_offset_and_length(
+                array.ends().clone(),
+                binary_numeric(array.values(), &rhs_const_array, op)?,
+                array.offset(),
+                array.len(),
+            )?
+            .into_array(),
+        ))
     }
 }
 
 #[cfg(test)]
 mod tests {
-    use vortex_array::array::PrimitiveArray;
+    use vortex_array::Array;
+    use vortex_array::arrays::PrimitiveArray;
     use vortex_array::compute::test_harness::test_binary_numeric;
-    use vortex_array::IntoArray;
 
     use crate::RunEndArray;
 
