@@ -1,27 +1,43 @@
-use vortex_array::array::PrimitiveArray;
-use vortex_array::stats::{Stat, StatsSet};
+use vortex_array::arrays::PrimitiveArray;
+use vortex_array::stats::{ArrayStats, Precision, Stat, StatsSet, StatsSetRef};
 use vortex_array::variants::PrimitiveArrayTrait;
-use vortex_array::visitor::ArrayVisitor;
-use vortex_array::vtable::{
-    CanonicalVTable, StatisticsVTable, ValidateVTable, ValidityVTable, VariantsVTable,
-    VisitorVTable,
-};
+use vortex_array::vtable::{EncodingVTable, StatisticsVTable, VTableRef};
 use vortex_array::{
-    encoding_ids, impl_encoding, Array, Canonical, EmptyMetadata, IntoArrayVariant,
+    Array, ArrayCanonicalImpl, ArrayImpl, ArrayRef, ArrayStatisticsImpl, ArrayValidityImpl,
+    ArrayVariantsImpl, Canonical, EmptyMetadata, Encoding, EncodingId, ToCanonical,
+    try_from_array_ref,
 };
 use vortex_dtype::{DType, PType};
-use vortex_error::{vortex_bail, vortex_err, vortex_panic, VortexExpect as _, VortexResult};
+use vortex_error::{VortexResult, vortex_bail, vortex_err};
 use vortex_mask::Mask;
-use vortex_scalar::ScalarValue;
 use zigzag::ZigZag as ExternalZigZag;
 
 use crate::compress::zigzag_encode;
 use crate::zigzag_decode;
 
-impl_encoding!("vortex.zigzag", encoding_ids::ZIGZAG, ZigZag, EmptyMetadata);
+#[derive(Clone, Debug)]
+pub struct ZigZagArray {
+    dtype: DType,
+    encoded: ArrayRef,
+    stats_set: ArrayStats,
+}
+
+try_from_array_ref!(ZigZagArray);
+
+pub struct ZigZagEncoding;
+impl Encoding for ZigZagEncoding {
+    type Array = ZigZagArray;
+    type Metadata = EmptyMetadata;
+}
+
+impl EncodingVTable for ZigZagEncoding {
+    fn id(&self) -> EncodingId {
+        EncodingId::new_ref("vortex.zigzag")
+    }
+}
 
 impl ZigZagArray {
-    pub fn try_new(encoded: Array) -> VortexResult<Self> {
+    pub fn try_new(encoded: ArrayRef) -> VortexResult<Self> {
         let encoded_dtype = encoded.dtype().clone();
         if !encoded_dtype.is_unsigned_int() {
             vortex_bail!(MismatchedTypes: "unsigned int", encoded_dtype);
@@ -30,73 +46,86 @@ impl ZigZagArray {
         let dtype = DType::from(PType::try_from(&encoded_dtype)?.to_signed())
             .with_nullability(encoded_dtype.nullability());
 
-        let len = encoded.len();
-        let children = [encoded];
-
-        Self::try_from_parts(
+        Ok(Self {
             dtype,
-            len,
-            EmptyMetadata,
-            None,
-            Some(children.into()),
-            StatsSet::default(),
-        )
+            encoded,
+            stats_set: Default::default(),
+        })
     }
 
-    pub fn encode(array: &Array) -> VortexResult<ZigZagArray> {
-        PrimitiveArray::try_from(array.clone())
+    pub fn encode(array: &dyn Array) -> VortexResult<ZigZagArray> {
+        PrimitiveArray::try_from(array.to_array())
             .map_err(|_| vortex_err!("ZigZag can only encoding primitive arrays"))
             .and_then(zigzag_encode)
     }
 
-    pub fn encoded(&self) -> Array {
-        let ptype = PType::try_from(self.dtype()).unwrap_or_else(|err| {
-            vortex_panic!(err, "Failed to convert DType {} to PType", self.dtype())
-        });
-        let encoded = DType::from(ptype.to_unsigned()).with_nullability(self.dtype().nullability());
-        self.as_ref()
-            .child(0, &encoded, self.len())
-            .vortex_expect("ZigZagArray is missing its encoded child array")
+    pub fn encoded(&self) -> &ArrayRef {
+        &self.encoded
     }
 }
 
-impl ValidateVTable<ZigZagArray> for ZigZagEncoding {}
+impl ArrayImpl for ZigZagArray {
+    type Encoding = ZigZagEncoding;
 
-impl VariantsVTable<ZigZagArray> for ZigZagEncoding {
-    fn as_primitive_array<'a>(
-        &self,
-        array: &'a ZigZagArray,
-    ) -> Option<&'a dyn PrimitiveArrayTrait> {
-        Some(array)
+    fn _len(&self) -> usize {
+        self.encoded.len()
+    }
+
+    fn _dtype(&self) -> &DType {
+        &self.dtype
+    }
+
+    fn _vtable(&self) -> VTableRef {
+        VTableRef::new_ref(&ZigZagEncoding)
+    }
+}
+
+impl ArrayCanonicalImpl for ZigZagArray {
+    fn _to_canonical(&self) -> VortexResult<Canonical> {
+        zigzag_decode(self.encoded().to_primitive()?).map(Canonical::Primitive)
+    }
+}
+
+impl ArrayStatisticsImpl for ZigZagArray {
+    fn _stats_ref(&self) -> StatsSetRef<'_> {
+        self.stats_set.to_ref(self)
+    }
+}
+
+impl ArrayValidityImpl for ZigZagArray {
+    fn _is_valid(&self, index: usize) -> VortexResult<bool> {
+        self.encoded.is_valid(index)
+    }
+
+    fn _all_valid(&self) -> VortexResult<bool> {
+        self.encoded.all_valid()
+    }
+
+    fn _all_invalid(&self) -> VortexResult<bool> {
+        self.encoded.all_invalid()
+    }
+
+    fn _validity_mask(&self) -> VortexResult<Mask> {
+        self.encoded.validity_mask()
+    }
+}
+
+impl ArrayVariantsImpl for ZigZagArray {
+    fn _as_primitive_typed(&self) -> Option<&dyn PrimitiveArrayTrait> {
+        Some(self)
     }
 }
 
 impl PrimitiveArrayTrait for ZigZagArray {}
 
-impl ValidityVTable<ZigZagArray> for ZigZagEncoding {
-    fn is_valid(&self, array: &ZigZagArray, index: usize) -> VortexResult<bool> {
-        array.encoded().is_valid(index)
-    }
-
-    fn logical_validity(&self, array: &ZigZagArray) -> VortexResult<Mask> {
-        array.encoded().logical_validity()
-    }
-}
-
-impl VisitorVTable<ZigZagArray> for ZigZagEncoding {
-    fn accept(&self, array: &ZigZagArray, visitor: &mut dyn ArrayVisitor) -> VortexResult<()> {
-        visitor.visit_child("encoded", &array.encoded())
-    }
-}
-
-impl StatisticsVTable<ZigZagArray> for ZigZagEncoding {
+impl StatisticsVTable<&ZigZagArray> for ZigZagEncoding {
     fn compute_statistics(&self, array: &ZigZagArray, stat: Stat) -> VortexResult<StatsSet> {
         let mut stats = StatsSet::default();
 
         // these stats are the same for array and array.encoded()
         if matches!(stat, Stat::IsConstant | Stat::NullCount) {
-            if let Some(val) = array.encoded().statistics().compute(stat) {
-                stats.set(stat, val);
+            if let Some(val) = array.encoded().statistics().compute_stat(stat)? {
+                stats.set(stat, Precision::exact(val));
             }
         } else if matches!(stat, Stat::Min | Stat::Max) {
             let encoded_max = array.encoded().statistics().compute_as::<u64>(Stat::Max);
@@ -104,7 +133,7 @@ impl StatisticsVTable<ZigZagArray> for ZigZagEncoding {
                 // the max of the encoded array is the element with the highest absolute value (so either min if negative, or max if positive)
                 let decoded = <i64 as ExternalZigZag>::decode(val);
                 let decoded_stat = if decoded < 0 { Stat::Min } else { Stat::Max };
-                stats.set(decoded_stat, ScalarValue::from(decoded));
+                stats.set(decoded_stat, Precision::exact(decoded));
             }
         }
 
@@ -112,16 +141,10 @@ impl StatisticsVTable<ZigZagArray> for ZigZagEncoding {
     }
 }
 
-impl CanonicalVTable<ZigZagArray> for ZigZagEncoding {
-    fn into_canonical(&self, array: ZigZagArray) -> VortexResult<Canonical> {
-        zigzag_decode(array.encoded().into_primitive()?).map(Canonical::Primitive)
-    }
-}
-
 #[cfg(test)]
 mod test {
-    use vortex_array::compute::{scalar_at, slice};
     use vortex_array::IntoArray;
+    use vortex_array::compute::{scalar_at, slice};
     use vortex_buffer::buffer;
     use vortex_scalar::Scalar;
 
@@ -145,7 +168,7 @@ mod test {
             array.statistics().compute_is_constant()
         );
 
-        let sliced = ZigZagArray::try_from(slice(zigzag, 0, 2).unwrap()).unwrap();
+        let sliced = ZigZagArray::try_from(slice(&zigzag, 0, 2).unwrap()).unwrap();
         assert_eq!(
             scalar_at(&sliced, sliced.len() - 1).unwrap(),
             Scalar::from(-5i32)
