@@ -1,6 +1,7 @@
+pub use array::*;
 pub use compress::*;
-pub use zigzag::*;
 
+mod array;
 mod compress;
 mod compute;
-mod zigzag;
+mod serde;
