@@ -2,11 +2,11 @@ import pyarrow as pa
 import pyarrow.compute as pc
 from substrait.proto import ExtendedExpression
 
-from .._lib import expr
-from ..substrait import extended_expression
+import vortex.expr
+from vortex.substrait import extended_expression
 
 
-def arrow_to_vortex(arrow_expression: pc.Expression, schema: pa.Schema) -> list[expr.Expr]:
+def arrow_to_vortex(arrow_expression: pc.Expression, schema: pa.Schema) -> vortex.expr.Expr:
     substrait_object = ExtendedExpression()
     substrait_object.ParseFromString(arrow_expression.to_substrait(schema))
 
