@@ -7,41 +7,51 @@ use futures::TryStreamExt;
 use pyo3::exceptions::PyTypeError;
 use pyo3::prelude::*;
 use pyo3::types::PyString;
-use vortex::array::ChunkedArray;
+use vortex::arrays::ChunkedArray;
 use vortex::arrow::infer_schema;
 use vortex::dtype::FieldName;
 use vortex::error::VortexResult;
-use vortex::expr::{ident, ExprRef, Select};
-use vortex::file::io::file::FileIoDriver;
-use vortex::file::read::VortexRecordBatchReader;
-use vortex::file::{Scan, VortexFile, VortexOpenOptions};
-use vortex::io::{ObjectStoreReadAt, TokioFile, VortexReadAt};
-use vortex::sampling_compressor::ALL_ENCODINGS_CONTEXT;
+use vortex::expr::{ExprRef, Select, ident};
+use vortex::file::{FileType, GenericVortexFile, VortexFile, VortexOpenOptions};
+use vortex::io::{ObjectStoreReadAt, TokioFile};
 use vortex::stream::ArrayStream;
-use vortex::{Array, IntoArray, IntoArrayVariant};
+use vortex::{Array, ArrayRef, ToCanonical};
 
+use crate::arrays::PyArray;
 use crate::expr::PyExpr;
 use crate::object_store_urls::vortex_read_at_from_url;
-use crate::{PyArray, TOKIO_RUNTIME};
+use crate::record_batch_reader::VortexRecordBatchReader;
+use crate::{TOKIO_RUNTIME, install_module};
 
-pub async fn read_array_from_reader<T: VortexReadAt + Unpin + 'static>(
-    vortex_file: &VortexFile<FileIoDriver<T>>,
+pub(crate) fn init(py: Python, parent: &Bound<PyModule>) -> PyResult<()> {
+    let m = PyModule::new(py, "dataset")?;
+    parent.add_submodule(&m)?;
+    install_module("vortex._lib.dataset", &m)?;
+
+    m.add_function(wrap_pyfunction!(dataset_from_url, &m)?)?;
+    m.add_function(wrap_pyfunction!(dataset_from_path, &m)?)?;
+
+    Ok(())
+}
+
+pub async fn read_array_from_reader<F: FileType>(
+    vortex_file: &VortexFile<F>,
     projection: ExprRef,
     filter: Option<ExprRef>,
-    indices: Option<Array>,
-) -> VortexResult<Array> {
-    let mut scan = Scan::new(projection);
+    indices: Option<ArrayRef>,
+) -> VortexResult<ArrayRef> {
+    let mut scan = vortex_file.scan().with_projection(projection);
 
     if let Some(filter) = filter {
         scan = scan.with_filter(filter);
     }
 
     if let Some(indices) = indices {
-        let indices = indices.into_primitive()?.into_buffer();
+        let indices = indices.to_primitive()?.into_buffer();
         scan = scan.with_row_indices(indices);
     }
 
-    let stream = vortex_file.scan(scan)?;
+    let stream = scan.into_array_stream()?;
     let dtype = stream.dtype().clone();
 
     let all_arrays = stream.try_collect::<Vec<_>>().await?;
@@ -83,16 +93,14 @@ fn filter_from_python(row_filter: Option<&Bound<PyExpr>>) -> Option<ExprRef> {
 
 #[pyclass(name = "TokioFileDataset", module = "io")]
 pub struct TokioFileDataset {
-    vxf: VortexFile<FileIoDriver<TokioFile>>,
+    vxf: VortexFile<GenericVortexFile<TokioFile>>,
     schema: SchemaRef,
 }
 
 impl TokioFileDataset {
     pub async fn try_new(path: String) -> VortexResult<Self> {
         let file = TokioFile::open(path)?;
-        let vxf = VortexOpenOptions::new(ALL_ENCODINGS_CONTEXT.clone())
-            .open(file)
-            .await?;
+        let vxf = VortexOpenOptions::file(file).open().await?;
         let schema = Arc::new(infer_schema(vxf.dtype())?);
 
         Ok(Self { vxf, schema })
@@ -103,15 +111,14 @@ impl TokioFileDataset {
         columns: Option<Vec<Bound<'_, PyAny>>>,
         row_filter: Option<&Bound<'_, PyExpr>>,
         indices: Option<&PyArray>,
-    ) -> PyResult<PyArray> {
-        let inner = read_array_from_reader(
+    ) -> PyResult<ArrayRef> {
+        Ok(read_array_from_reader(
             &self.vxf,
             projection_from_python(columns)?,
             filter_from_python(row_filter),
-            indices.map(PyArray::unwrap).cloned(),
+            indices.cloned().map(PyArray::into_inner),
         )
-        .await?;
-        Ok(PyArray::new(inner))
+        .await?)
     }
 
     async fn async_to_record_batch_reader(
@@ -120,18 +127,18 @@ impl TokioFileDataset {
         row_filter: Option<&Bound<'_, PyExpr>>,
         indices: Option<&PyArray>,
     ) -> PyResult<PyObject> {
-        let mut scan = Scan::new(projection_from_python(columns)?);
+        let mut scan = self_
+            .vxf
+            .scan()
+            .with_projection(projection_from_python(columns)?)
+            .with_some_filter(filter_from_python(row_filter));
 
-        if let Some(filter) = filter_from_python(row_filter) {
-            scan = scan.with_filter(filter);
-        }
-
-        if let Some(indices) = indices.map(PyArray::unwrap).cloned() {
-            let indices = indices.into_primitive()?.into_buffer();
+        if let Some(indices) = indices.cloned().map(PyArray::into_inner) {
+            let indices = indices.to_primitive()?.into_buffer();
             scan = scan.with_row_indices(indices);
         }
 
-        let stream = self_.vxf.scan(scan)?;
+        let stream = scan.into_array_stream()?;
 
         let record_batch_reader: Box<dyn RecordBatchReader + Send> =
             Box::new(VortexRecordBatchReader::try_new(stream, &*TOKIO_RUNTIME)?);
@@ -147,13 +154,17 @@ impl TokioFileDataset {
     }
 
     #[pyo3(signature = (*, columns = None, row_filter = None, indices = None))]
-    pub fn to_array(
+    pub fn to_array<'py>(
         &self,
-        columns: Option<Vec<Bound<'_, PyAny>>>,
-        row_filter: Option<&Bound<'_, PyExpr>>,
+        py: Python<'py>,
+        columns: Option<Vec<Bound<'py, PyAny>>>,
+        row_filter: Option<&Bound<'py, PyExpr>>,
         indices: Option<&PyArray>,
-    ) -> PyResult<PyArray> {
-        TOKIO_RUNTIME.block_on(self.async_to_array(columns, row_filter, indices))
+    ) -> PyResult<Bound<'py, PyArray>> {
+        PyArray::init(
+            py,
+            TOKIO_RUNTIME.block_on(self.async_to_array(columns, row_filter, indices))?,
+        )
     }
 
     #[pyo3(signature = (*, columns = None, row_filter = None, indices = None))]
@@ -171,7 +182,7 @@ impl TokioFileDataset {
 
 #[pyclass(name = "ObjectStoreUrlDataset", module = "io")]
 pub struct ObjectStoreUrlDataset {
-    vxf: VortexFile<FileIoDriver<ObjectStoreReadAt>>,
+    vxf: VortexFile<GenericVortexFile<ObjectStoreReadAt>>,
     schema: SchemaRef,
 }
 
@@ -179,9 +190,7 @@ impl ObjectStoreUrlDataset {
     pub async fn try_new(url: String) -> VortexResult<Self> {
         let reader = vortex_read_at_from_url(&url).await?;
 
-        let vxf = VortexOpenOptions::new(ALL_ENCODINGS_CONTEXT.clone())
-            .open(reader)
-            .await?;
+        let vxf = VortexOpenOptions::file(reader).open().await?;
         let schema = Arc::new(infer_schema(vxf.dtype())?);
 
         Ok(Self { vxf, schema })
@@ -192,15 +201,14 @@ impl ObjectStoreUrlDataset {
         columns: Option<Vec<Bound<'_, PyAny>>>,
         row_filter: Option<&Bound<'_, PyExpr>>,
         indices: Option<&PyArray>,
-    ) -> PyResult<PyArray> {
-        let inner = read_array_from_reader(
+    ) -> PyResult<ArrayRef> {
+        Ok(read_array_from_reader(
             &self.vxf,
             projection_from_python(columns)?,
             filter_from_python(row_filter),
-            indices.map(PyArray::unwrap).cloned(),
+            indices.cloned().map(PyArray::into_inner),
         )
-        .await?;
-        Ok(PyArray::new(inner))
+        .await?)
     }
 
     async fn async_to_record_batch_reader(
@@ -209,18 +217,18 @@ impl ObjectStoreUrlDataset {
         filter: Option<&Bound<'_, PyExpr>>,
         indices: Option<&PyArray>,
     ) -> PyResult<PyObject> {
-        let mut scan = Scan::new(projection_from_python(columns)?);
+        let mut scan = self_
+            .vxf
+            .scan()
+            .with_projection(projection_from_python(columns)?)
+            .with_some_filter(filter_from_python(filter));
 
-        if let Some(filter) = filter_from_python(filter) {
-            scan = scan.with_filter(filter);
-        }
-
-        if let Some(indices) = indices.map(PyArray::unwrap).cloned() {
-            let indices = indices.into_primitive()?.into_buffer();
+        if let Some(indices) = indices.cloned().map(PyArray::into_inner) {
+            let indices = indices.to_primitive()?.into_buffer();
             scan = scan.with_row_indices(indices);
         }
 
-        let stream = self_.vxf.scan(scan)?;
+        let stream = scan.into_array_stream()?;
 
         let record_batch_reader: Box<dyn RecordBatchReader + Send> =
             Box::new(VortexRecordBatchReader::try_new(stream, &*TOKIO_RUNTIME)?);
@@ -235,13 +243,17 @@ impl ObjectStoreUrlDataset {
     }
 
     #[pyo3(signature = (*, columns = None, row_filter = None, indices = None))]
-    pub fn to_array(
+    pub fn to_array<'py>(
         &self,
-        columns: Option<Vec<Bound<'_, PyAny>>>,
-        row_filter: Option<&Bound<'_, PyExpr>>,
+        py: Python<'py>,
+        columns: Option<Vec<Bound<'py, PyAny>>>,
+        row_filter: Option<&Bound<'py, PyExpr>>,
         indices: Option<&PyArray>,
-    ) -> PyResult<PyArray> {
-        TOKIO_RUNTIME.block_on(self.async_to_array(columns, row_filter, indices))
+    ) -> PyResult<Bound<'py, PyArray>> {
+        PyArray::init(
+            py,
+            TOKIO_RUNTIME.block_on(self.async_to_array(columns, row_filter, indices))?,
+        )
     }
 
     #[pyo3(signature = (*, columns = None, row_filter = None, indices = None))]
