@@ -3,14 +3,14 @@ use arrow_array::array::{
     BooleanArray as ArrowBooleanArray, GenericByteArray, NullArray as ArrowNullArray,
     OffsetSizeTrait, PrimitiveArray as ArrowPrimitiveArray, StructArray as ArrowStructArray,
 };
-use arrow_array::cast::{as_null_array, AsArray};
+use arrow_array::cast::{AsArray, as_null_array};
 use arrow_array::types::{
     ByteArrayType, ByteViewType, Date32Type, Date64Type, DurationMicrosecondType,
     DurationMillisecondType, DurationNanosecondType, DurationSecondType, Float16Type, Float32Type,
-    Float64Type, Int16Type, Int32Type, Int64Type, Int8Type, Time32MillisecondType,
+    Float64Type, Int8Type, Int16Type, Int32Type, Int64Type, Time32MillisecondType,
     Time32SecondType, Time64MicrosecondType, Time64NanosecondType, TimestampMicrosecondType,
-    TimestampMillisecondType, TimestampNanosecondType, TimestampSecondType, UInt16Type, UInt32Type,
-    UInt64Type, UInt8Type,
+    TimestampMillisecondType, TimestampNanosecondType, TimestampSecondType, UInt8Type, UInt16Type,
+    UInt32Type, UInt64Type,
 };
 use arrow_array::{BinaryViewArray, GenericByteViewArray, GenericListArray, StringViewArray};
 use arrow_buffer::buffer::{NullBuffer, OffsetBuffer};
@@ -18,22 +18,21 @@ use arrow_buffer::{ArrowNativeType, BooleanBuffer, Buffer as ArrowBuffer, Scalar
 use arrow_schema::{DataType, TimeUnit as ArrowTimeUnit};
 use vortex_buffer::{Alignment, Buffer, ByteBuffer};
 use vortex_datetime_dtype::TimeUnit;
-use vortex_dtype::{DType, NativePType, Nullability, PType};
-use vortex_error::{vortex_panic, VortexExpect as _};
+use vortex_dtype::{DType, NativePType, PType};
+use vortex_error::{VortexExpect as _, vortex_panic};
 
-use crate::array::{
+use crate::arrays::{
     BoolArray, ListArray, NullArray, PrimitiveArray, StructArray, TemporalArray, VarBinArray,
     VarBinViewArray,
 };
 use crate::arrow::FromArrowArray;
-use crate::stats::Stat;
 use crate::validity::Validity;
-use crate::{Array, IntoArray};
+use crate::{Array, ArrayRef, IntoArray};
 
-impl From<ArrowBuffer> for Array {
-    fn from(value: ArrowBuffer) -> Self {
+impl IntoArray for ArrowBuffer {
+    fn into_array(self) -> ArrayRef {
         PrimitiveArray::from_byte_buffer(
-            ByteBuffer::from_arrow_buffer(value, Alignment::of::<u8>()),
+            ByteBuffer::from_arrow_buffer(self, Alignment::of::<u8>()),
             PType::U8,
             Validity::NonNullable,
         )
@@ -41,43 +40,41 @@ impl From<ArrowBuffer> for Array {
     }
 }
 
-impl From<BooleanBuffer> for Array {
-    fn from(value: BooleanBuffer) -> Self {
-        BoolArray::new(value, Nullability::NonNullable).into_array()
+impl IntoArray for BooleanBuffer {
+    fn into_array(self) -> ArrayRef {
+        BoolArray::new(self, Validity::NonNullable).into_array()
     }
 }
 
-impl<T> From<ScalarBuffer<T>> for Array
+impl<T> IntoArray for ScalarBuffer<T>
 where
     T: ArrowNativeType + NativePType,
 {
-    fn from(value: ScalarBuffer<T>) -> Self {
+    fn into_array(self) -> ArrayRef {
         PrimitiveArray::new(
-            Buffer::<T>::from_arrow_scalar_buffer(value),
+            Buffer::<T>::from_arrow_scalar_buffer(self),
             Validity::NonNullable,
         )
         .into_array()
     }
 }
 
-impl<O> From<OffsetBuffer<O>> for Array
+impl<O> IntoArray for OffsetBuffer<O>
 where
     O: NativePType + OffsetSizeTrait,
 {
-    fn from(value: OffsetBuffer<O>) -> Self {
+    fn into_array(self) -> ArrayRef {
         let primitive = PrimitiveArray::new(
-            Buffer::from_arrow_scalar_buffer(value.into_inner()),
+            Buffer::from_arrow_scalar_buffer(self.into_inner()),
             Validity::NonNullable,
         );
-        primitive.statistics().set(Stat::IsSorted, true.into());
-        primitive
-            .statistics()
-            .set(Stat::IsStrictSorted, true.into());
+        // primitive.update_statistic(Stat::IsSorted, Precision::exact(true));
+        // primitive.update_statistic(Stat::IsStrictSorted, Precision::exact(true));
         primitive.into_array()
     }
 }
 
-impl<T: ArrowPrimitiveType> FromArrowArray<&ArrowPrimitiveArray<T>> for Array
+impl<T: ArrowPrimitiveType> FromArrowArray<&ArrowPrimitiveArray<T>> for ArrayRef
 where
     <T as ArrowPrimitiveType>::Native: NativePType,
 {
@@ -111,7 +108,7 @@ where
     }
 }
 
-impl<T: ByteArrayType> FromArrowArray<&GenericByteArray<T>> for Array
+impl<T: ByteArrayType> FromArrowArray<&GenericByteArray<T>> for ArrayRef
 where
     <T as ByteArrayType>::Offset: NativePType,
 {
@@ -122,7 +119,7 @@ where
             _ => vortex_panic!("Invalid data type for ByteArray: {}", T::DATA_TYPE),
         };
         VarBinArray::try_new(
-            value.offsets().clone().into(),
+            value.offsets().clone().into_array(),
             ByteBuffer::from_arrow_buffer(value.values().clone(), Alignment::of::<u8>()),
             dtype,
             nulls(value.nulls(), nullable),
@@ -132,7 +129,7 @@ where
     }
 }
 
-impl<T: ByteViewType> FromArrowArray<&GenericByteViewArray<T>> for Array {
+impl<T: ByteViewType> FromArrowArray<&GenericByteViewArray<T>> for ArrayRef {
     fn from_arrow(value: &GenericByteViewArray<T>, nullable: bool) -> Self {
         let dtype = match T::DATA_TYPE {
             DataType::BinaryView => DType::Binary(nullable.into()),
@@ -159,15 +156,13 @@ impl<T: ByteViewType> FromArrowArray<&GenericByteViewArray<T>> for Array {
     }
 }
 
-impl FromArrowArray<&ArrowBooleanArray> for Array {
+impl FromArrowArray<&ArrowBooleanArray> for ArrayRef {
     fn from_arrow(value: &ArrowBooleanArray, nullable: bool) -> Self {
-        BoolArray::try_new(value.values().clone(), nulls(value.nulls(), nullable))
-            .vortex_expect("Validity length cannot mismatch")
-            .into_array()
+        BoolArray::new(value.values().clone(), nulls(value.nulls(), nullable)).into_array()
     }
 }
 
-impl FromArrowArray<&ArrowStructArray> for Array {
+impl FromArrowArray<&ArrowStructArray> for ArrayRef {
     fn from_arrow(value: &ArrowStructArray, nullable: bool) -> Self {
         StructArray::try_new(
             value.column_names().iter().map(|s| (*s).into()).collect(),
@@ -185,7 +180,7 @@ impl FromArrowArray<&ArrowStructArray> for Array {
     }
 }
 
-impl<O: OffsetSizeTrait + NativePType> FromArrowArray<&GenericListArray<O>> for Array {
+impl<O: OffsetSizeTrait + NativePType> FromArrowArray<&GenericListArray<O>> for ArrayRef {
     fn from_arrow(value: &GenericListArray<O>, nullable: bool) -> Self {
         // Extract the validity of the underlying element array
         let elem_nullable = match value.data_type() {
@@ -196,7 +191,7 @@ impl<O: OffsetSizeTrait + NativePType> FromArrowArray<&GenericListArray<O>> for
         ListArray::try_new(
             Self::from_arrow(value.values().clone(), elem_nullable),
             // offsets are always non-nullable
-            Array::from(value.offsets().clone()),
+            value.offsets().clone().into_array(),
             nulls(value.nulls(), nullable),
         )
         .vortex_expect("Failed to convert Arrow StructArray to Vortex StructArray")
@@ -204,7 +199,7 @@ impl<O: OffsetSizeTrait + NativePType> FromArrowArray<&GenericListArray<O>> for
     }
 }
 
-impl FromArrowArray<&ArrowNullArray> for Array {
+impl FromArrowArray<&ArrowNullArray> for ArrayRef {
     fn from_arrow(value: &ArrowNullArray, nullable: bool) -> Self {
         assert!(nullable);
         NullArray::new(value.len()).into_array()
@@ -228,7 +223,7 @@ fn nulls(nulls: Option<&NullBuffer>, nullable: bool) -> Validity {
     }
 }
 
-impl FromArrowArray<ArrowArrayRef> for Array {
+impl FromArrowArray<ArrowArrayRef> for ArrayRef {
     fn from_arrow(array: ArrowArrayRef, nullable: bool) -> Self {
         match array.data_type() {
             DataType::Boolean => Self::from_arrow(array.as_boolean(), nullable),
