@@ -1,5 +1,7 @@
 //! Utilities to work with `Arrow` data and types.
 
+use arrow_array::ArrayRef;
+use arrow_schema::DataType;
 use vortex_error::VortexResult;
 
 pub use crate::arrow::dtype::{infer_data_type, infer_schema};
@@ -7,8 +9,7 @@ pub use crate::arrow::dtype::{infer_data_type, infer_schema};
 mod array;
 mod datum;
 mod dtype;
-mod recordbatch;
-pub mod wrappers;
+mod record_batch;
 pub use datum::*;
 
 pub trait FromArrowArray<A> {
@@ -22,3 +23,9 @@ pub trait FromArrowType<T>: Sized {
 pub trait TryFromArrowType<T>: Sized {
     fn try_from_arrow(value: T) -> VortexResult<Self>;
 }
+
+pub trait IntoArrowArray {
+    fn into_arrow_preferred(self) -> VortexResult<ArrayRef>;
+
+    fn into_arrow(self, data_type: &DataType) -> VortexResult<ArrayRef>;
+}
