@@ -1,29 +1,30 @@
-use vortex_error::{vortex_bail, vortex_err, VortexError, VortexResult};
+use vortex_error::{VortexExpect, VortexResult, vortex_bail, vortex_err};
 use vortex_scalar::Scalar;
 
-use crate::encoding::Encoding;
 use crate::Array;
+use crate::encoding::Encoding;
 
 /// Implementation of scalar_at for an encoding.
 ///
-/// SAFETY: the index is guaranteed to be within the bounds of the [Array].
+/// SAFETY: the index is guaranteed to be within the bounds of the [`crate::ArrayRef`].
 pub trait ScalarAtFn<A> {
-    fn scalar_at(&self, array: &A, index: usize) -> VortexResult<Scalar>;
+    fn scalar_at(&self, array: A, index: usize) -> VortexResult<Scalar>;
 }
 
-impl<E: Encoding> ScalarAtFn<Array> for E
+impl<E: Encoding> ScalarAtFn<&dyn Array> for E
 where
-    E: ScalarAtFn<E::Array>,
-    for<'a> &'a E::Array: TryFrom<&'a Array, Error = VortexError>,
+    E: for<'a> ScalarAtFn<&'a E::Array>,
 {
-    fn scalar_at(&self, array: &Array, index: usize) -> VortexResult<Scalar> {
-        let (array_ref, encoding) = array.try_downcast_ref::<E>()?;
-        ScalarAtFn::scalar_at(encoding, array_ref, index)
+    fn scalar_at(&self, array: &dyn Array, index: usize) -> VortexResult<Scalar> {
+        let array_ref = array
+            .as_any()
+            .downcast_ref::<E::Array>()
+            .vortex_expect("Failed to downcast array");
+        ScalarAtFn::scalar_at(self, array_ref, index)
     }
 }
 
-pub fn scalar_at(array: impl AsRef<Array>, index: usize) -> VortexResult<Scalar> {
-    let array = array.as_ref();
+pub fn scalar_at(array: &dyn Array, index: usize) -> VortexResult<Scalar> {
     if index >= array.len() {
         vortex_bail!(OutOfBounds: index, 0, array.len());
     }
