@@ -1,77 +1,18 @@
 //! Array validity and nullability behavior, used by arrays and compute functions.
 
-use std::fmt::{Debug, Display};
-use std::ops::BitAnd;
+use std::fmt::Debug;
+use std::ops::{BitAnd, Not};
 
 use arrow_buffer::{BooleanBuffer, BooleanBufferBuilder, NullBuffer};
-use serde::{Deserialize, Serialize};
 use vortex_dtype::{DType, Nullability};
-use vortex_error::{vortex_bail, vortex_err, vortex_panic, VortexExpect as _, VortexResult};
-use vortex_mask::{Mask, MaskValues};
+use vortex_error::{VortexExpect as _, VortexResult, vortex_bail, vortex_err, vortex_panic};
+use vortex_mask::{AllOr, Mask, MaskValues};
+use vortex_scalar::Scalar;
 
-use crate::array::{BoolArray, ConstantArray};
-use crate::compute::{filter, scalar_at, slice, take};
+use crate::arrays::{BoolArray, ConstantArray};
+use crate::compute::{fill_null, filter, scalar_at, slice, take};
 use crate::patches::Patches;
-use crate::{Array, IntoArray, IntoArrayVariant};
-
-impl Array {
-    /// Return whether the element at the given index is valid (true) or null (false).
-    pub fn is_valid(&self, index: usize) -> VortexResult<bool> {
-        self.vtable().is_valid(self, index)
-    }
-
-    /// Return the number of null elements in the array.
-    pub fn null_count(&self) -> VortexResult<usize> {
-        self.vtable().null_count(self)
-    }
-
-    /// Return the logical validity of the array if nullable, and None if non-nullable.
-    pub fn logical_validity(&self) -> VortexResult<Mask> {
-        self.vtable().logical_validity(self)
-    }
-}
-
-#[derive(
-    Copy,
-    Clone,
-    Debug,
-    Serialize,
-    Deserialize,
-    rkyv::Archive,
-    rkyv::Portable,
-    rkyv::Serialize,
-    rkyv::Deserialize,
-    rkyv::bytecheck::CheckBytes,
-)]
-#[rkyv(as = ValidityMetadata)]
-#[bytecheck(crate = rkyv::bytecheck)]
-#[repr(u8)]
-pub enum ValidityMetadata {
-    NonNullable,
-    AllValid,
-    AllInvalid,
-    Array,
-}
-
-impl Display for ValidityMetadata {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        Debug::fmt(self, f)
-    }
-}
-
-impl ValidityMetadata {
-    pub fn to_validity<F>(&self, array_fn: F) -> Validity
-    where
-        F: FnOnce() -> Array,
-    {
-        match self {
-            Self::NonNullable => Validity::NonNullable,
-            Self::AllValid => Validity::AllValid,
-            Self::AllInvalid => Validity::AllInvalid,
-            Self::Array => Validity::Array(array_fn()),
-        }
-    }
-}
+use crate::{Array, ArrayRef, ArrayVariants, IntoArray, ToCanonical};
 
 /// Validity information for an array
 #[derive(Clone, Debug)]
@@ -83,33 +24,13 @@ pub enum Validity {
     /// All items are null
     AllInvalid,
     /// Specified items are null
-    Array(Array),
+    Array(ArrayRef),
 }
 
 impl Validity {
     /// The [`DType`] of the underlying validity array (if it exists).
     pub const DTYPE: DType = DType::Bool(Nullability::NonNullable);
 
-    pub fn to_metadata(&self, length: usize) -> VortexResult<ValidityMetadata> {
-        match self {
-            Self::NonNullable => Ok(ValidityMetadata::NonNullable),
-            Self::AllValid => Ok(ValidityMetadata::AllValid),
-            Self::AllInvalid => Ok(ValidityMetadata::AllInvalid),
-            Self::Array(a) => {
-                // We force the caller to validate the length here.
-                let validity_len = a.len();
-                if validity_len != length {
-                    vortex_bail!(
-                        "Validity array length {} doesn't match array length {}",
-                        validity_len,
-                        length
-                    )
-                }
-                Ok(ValidityMetadata::Array)
-            }
-        }
-    }
-
     pub fn null_count(&self, length: usize) -> VortexResult<usize> {
         match self {
             Self::NonNullable | Self::AllValid => Ok(0),
@@ -123,16 +44,17 @@ impl Validity {
                         length
                     )
                 }
-                let true_count = a.statistics().compute_true_count().ok_or_else(|| {
-                    vortex_err!("Failed to compute true count from validity array")
-                })?;
+                let true_count = a
+                    .as_bool_typed()
+                    .vortex_expect("Validity array must be boolean")
+                    .true_count()?;
                 Ok(length - true_count)
             }
         }
     }
 
     /// If Validity is [`Validity::Array`], returns the array, otherwise returns `None`.
-    pub fn into_array(self) -> Option<Array> {
+    pub fn into_array(self) -> Option<ArrayRef> {
         match self {
             Self::Array(a) => Some(a),
             _ => None,
@@ -140,7 +62,7 @@ impl Validity {
     }
 
     /// If Validity is [`Validity::Array`], returns a reference to the array array, otherwise returns `None`.
-    pub fn as_array(&self) -> Option<&Array> {
+    pub fn as_array(&self) -> Option<&ArrayRef> {
         match self {
             Self::Array(a) => Some(a),
             _ => None,
@@ -154,6 +76,28 @@ impl Validity {
         }
     }
 
+    pub fn all_valid(&self) -> VortexResult<bool> {
+        Ok(match self {
+            Validity::NonNullable | Validity::AllValid => true,
+            Validity::AllInvalid => false,
+            Validity::Array(array) => {
+                // TODO(ngates): replace with SUM compute function
+                array.to_bool()?.boolean_buffer().count_set_bits() == array.len()
+            }
+        })
+    }
+
+    pub fn all_invalid(&self) -> VortexResult<bool> {
+        Ok(match self {
+            Validity::NonNullable | Validity::AllValid => false,
+            Validity::AllInvalid => true,
+            Validity::Array(array) => {
+                // TODO(ngates): replace with SUM compute function
+                array.to_bool()?.boolean_buffer().count_set_bits() == 0
+            }
+        })
+    }
+
     /// Returns whether the `index` item is valid.
     #[inline]
     pub fn is_valid(&self, index: usize) -> VortexResult<bool> {
@@ -182,40 +126,37 @@ impl Validity {
         }
     }
 
-    pub fn take(&self, indices: &Array) -> VortexResult<Self> {
+    pub fn take(&self, indices: &dyn Array) -> VortexResult<Self> {
         match self {
-            Self::NonNullable => Ok(Self::NonNullable),
-            Self::AllValid => Ok(Self::AllValid),
+            Self::NonNullable => match indices.validity_mask()?.boolean_buffer() {
+                AllOr::All => {
+                    if indices.dtype().is_nullable() {
+                        Ok(Self::AllValid)
+                    } else {
+                        Ok(Self::NonNullable)
+                    }
+                }
+                AllOr::None => Ok(Self::AllInvalid),
+                AllOr::Some(buf) => Ok(Validity::from(buf.clone())),
+            },
+            Self::AllValid => match indices.validity_mask()?.boolean_buffer() {
+                AllOr::All => Ok(Self::AllValid),
+                AllOr::None => Ok(Self::AllInvalid),
+                AllOr::Some(buf) => Ok(Validity::from(buf.clone())),
+            },
             Self::AllInvalid => Ok(Self::AllInvalid),
-            Self::Array(a) => Ok(Self::Array(take(a, indices)?)),
-        }
-    }
-
-    /// Take the validity buffer at the provided indices.
-    ///
-    /// # Safety
-    ///
-    /// It is assumed the caller has checked that all indices are <= the length of this validity
-    /// buffer.
-    ///
-    /// Failure to do so may result in UB.
-    pub unsafe fn take_unchecked(&self, indices: &Array) -> VortexResult<Self> {
-        match self {
-            Self::NonNullable => Ok(Self::NonNullable),
-            Self::AllValid => Ok(Self::AllValid),
-            Self::AllInvalid => Ok(Self::AllInvalid),
-            Self::Array(a) => {
-                let taken = if let Some(take_fn) = a.vtable().take_fn() {
-                    unsafe { take_fn.take_unchecked(a, indices) }
-                } else {
-                    take(a, indices)
-                };
-
-                taken.map(Self::Array)
+            Self::Array(is_valid) => {
+                let maybe_is_valid = take(is_valid, indices)?;
+                // Null indices invalidite that position.
+                let is_valid = fill_null(&maybe_is_valid, Scalar::from(false))?;
+                Ok(Self::Array(is_valid))
             }
         }
     }
 
+    /// Keep only the entries for which the mask is true.
+    ///
+    /// The result has length equal to the number of true values in mask.
     pub fn filter(&self, mask: &Mask) -> VortexResult<Self> {
         // NOTE(ngates): we take the mask as a reference to avoid the caller cloning unnecessarily
         //  if we happen to be NonNullable, AllValid, or AllInvalid.
@@ -227,11 +168,42 @@ impl Validity {
         }
     }
 
+    /// Set to false any entries for which the mask is true.
+    ///
+    /// The result is always nullable. The result has the same length as self.
+    pub fn mask(&self, mask: &Mask) -> VortexResult<Self> {
+        match mask.boolean_buffer() {
+            AllOr::All => Ok(Validity::AllInvalid),
+            AllOr::None => Ok(self.clone()),
+            AllOr::Some(make_invalid) => Ok(match self {
+                Validity::NonNullable | Validity::AllValid => {
+                    Validity::Array(BoolArray::from(make_invalid.not()).into_array())
+                }
+                Validity::AllInvalid => Validity::AllInvalid,
+                Validity::Array(is_valid) => {
+                    let is_valid = is_valid.to_bool()?;
+                    let keep_valid = make_invalid.not();
+                    Validity::from(is_valid.boolean_buffer().bitand(&keep_valid))
+                }
+            }),
+        }
+    }
+
+    // TODO(ngates): rename to to_mask
     pub fn to_logical(&self, length: usize) -> VortexResult<Mask> {
         Ok(match self {
             Self::NonNullable | Self::AllValid => Mask::AllTrue(length),
             Self::AllInvalid => Mask::AllFalse(length),
-            Self::Array(a) => Mask::try_from(a.clone())?,
+            Self::Array(is_valid) => {
+                assert_eq!(
+                    is_valid.len(),
+                    length,
+                    "Validity::Array length must equal to_logical's argument: {}, {}.",
+                    is_valid.len(),
+                    length,
+                );
+                Mask::try_from(&is_valid.to_bool()?)?
+            }
         })
     }
 
@@ -253,21 +225,27 @@ impl Validity {
             | (Validity::AllValid, Validity::AllValid) => Validity::AllValid,
             // Here we actually have to do some work
             (Validity::Array(lhs), Validity::Array(rhs)) => {
-                let lhs = BoolArray::try_from(lhs)?;
-                let rhs = BoolArray::try_from(rhs)?;
+                let lhs = lhs.to_bool()?;
+                let rhs = rhs.to_bool()?;
 
                 let lhs = lhs.boolean_buffer();
                 let rhs = rhs.boolean_buffer();
 
-                Validity::from(lhs.bitand(&rhs))
+                Validity::from(lhs.bitand(rhs))
             }
         };
 
         Ok(validity)
     }
 
-    pub fn patch(self, len: usize, indices: &Array, patches: Validity) -> VortexResult<Self> {
-        match (&self, &patches) {
+    pub fn patch(
+        self,
+        len: usize,
+        indices_offset: usize,
+        indices: &dyn Array,
+        patches: &Validity,
+    ) -> VortexResult<Self> {
+        match (&self, patches) {
             (Validity::NonNullable, Validity::NonNullable) => return Ok(Validity::NonNullable),
             (Validity::NonNullable, _) => {
                 vortex_bail!("Can't patch a non-nullable validity with nullable validity")
@@ -290,20 +268,25 @@ impl Validity {
             Validity::NonNullable => BoolArray::from(BooleanBuffer::new_set(len)),
             Validity::AllValid => BoolArray::from(BooleanBuffer::new_set(len)),
             Validity::AllInvalid => BoolArray::from(BooleanBuffer::new_unset(len)),
-            Validity::Array(a) => a.into_bool()?,
+            Validity::Array(a) => a.to_bool()?,
         };
 
         let patch_values = match patches {
             Validity::NonNullable => BoolArray::from(BooleanBuffer::new_set(indices.len())),
             Validity::AllValid => BoolArray::from(BooleanBuffer::new_set(indices.len())),
             Validity::AllInvalid => BoolArray::from(BooleanBuffer::new_unset(indices.len())),
-            Validity::Array(a) => a.into_bool()?,
+            Validity::Array(a) => a.to_bool()?,
         };
 
-        let patches = Patches::new(len, indices.clone(), patch_values.into_array());
+        let patches = Patches::new(
+            len,
+            indices_offset,
+            indices.to_array(),
+            patch_values.into_array(),
+        );
 
         Ok(Self::from_array(
-            source.patch(patches)?.into_array(),
+            source.patch(&patches)?.into_array(),
             own_nullability,
         ))
     }
@@ -316,11 +299,48 @@ impl Validity {
         }
     }
 
+    /// Convert into a non-nullable variant
+    pub fn into_non_nullable(self) -> Option<Validity> {
+        match self {
+            Self::NonNullable => Some(Self::NonNullable),
+            Self::AllValid => Some(Self::NonNullable),
+            Self::AllInvalid => None,
+            Self::Array(is_valid) => {
+                is_valid
+                    .statistics()
+                    .compute_min::<bool>()
+                    .vortex_expect("validity array must support min")
+                    .then(|| {
+                        // min true => all true
+                        Self::NonNullable
+                    })
+            }
+        }
+    }
+
+    /// Convert into a variant compatible with the given nullability, if possible.
+    pub fn cast_nullability(self, nullability: Nullability) -> VortexResult<Validity> {
+        match nullability {
+            Nullability::NonNullable => self.into_non_nullable().ok_or_else(|| {
+                vortex_err!("Cannot cast array with invalid values to non-nullable type.")
+            }),
+            Nullability::Nullable => Ok(self.into_nullable()),
+        }
+    }
+
+    /// Create Validity by copying the given array's validity.
+    pub fn copy_from_array(array: &dyn Array) -> VortexResult<Self> {
+        Ok(Validity::from_mask(
+            array.validity_mask()?,
+            array.dtype().nullability(),
+        ))
+    }
+
     /// Create Validity from boolean array with given nullability of the array.
     ///
     /// Note: You want to pass the nullability of parent array and not the nullability of the validity array itself
     ///     as that is always nonnullable
-    pub fn from_array(value: Array, nullability: Nullability) -> Self {
+    fn from_array(value: ArrayRef, nullability: Nullability) -> Self {
         if !matches!(value.dtype(), DType::Bool(Nullability::NonNullable)) {
             vortex_panic!("Expected a non-nullable boolean array")
         }
@@ -329,6 +349,22 @@ impl Validity {
             Nullability::Nullable => Self::Array(value),
         }
     }
+
+    /// Returns the length of the validity array, if it exists.
+    pub fn maybe_len(&self) -> Option<usize> {
+        match self {
+            Self::NonNullable | Self::AllValid | Self::AllInvalid => None,
+            Self::Array(a) => Some(a.len()),
+        }
+    }
+
+    pub fn uncompressed_size(&self) -> usize {
+        if let Validity::Array(a) = self {
+            a.len().div_ceil(8)
+        } else {
+            0
+        }
+    }
 }
 
 impl PartialEq for Validity {
@@ -338,17 +374,13 @@ impl PartialEq for Validity {
             (Self::AllValid, Self::AllValid) => true,
             (Self::AllInvalid, Self::AllInvalid) => true,
             (Self::Array(a), Self::Array(b)) => {
-                let a_buffer = a
-                    .clone()
-                    .into_bool()
-                    .vortex_expect("Failed to get Validity Array as BoolArray")
-                    .boolean_buffer();
-                let b_buffer = b
-                    .clone()
-                    .into_bool()
-                    .vortex_expect("Failed to get Validity Array as BoolArray")
-                    .boolean_buffer();
-                a_buffer == b_buffer
+                let a = a
+                    .to_bool()
+                    .vortex_expect("Failed to get Validity Array as BoolArray");
+                let b = b
+                    .to_bool()
+                    .vortex_expect("Failed to get Validity Array as BoolArray");
+                a.boolean_buffer() == b.boolean_buffer()
             }
             _ => false,
         }
@@ -435,7 +467,7 @@ impl Validity {
 }
 
 impl IntoArray for Mask {
-    fn into_array(self) -> Array {
+    fn into_array(self) -> ArrayRef {
         match self {
             Self::AllTrue(len) => ConstantArray::new(true, len).into_array(),
             Self::AllFalse(len) => ConstantArray::new(false, len).into_array(),
@@ -445,21 +477,22 @@ impl IntoArray for Mask {
 }
 
 impl IntoArray for &MaskValues {
-    fn into_array(self) -> Array {
-        BoolArray::new(self.boolean_buffer().clone(), Nullability::NonNullable).into_array()
+    fn into_array(self) -> ArrayRef {
+        BoolArray::new(self.boolean_buffer().clone(), Validity::NonNullable).into_array()
     }
 }
 
 #[cfg(test)]
 mod tests {
     use rstest::rstest;
-    use vortex_buffer::{buffer, Buffer};
+    use vortex_buffer::{Buffer, buffer};
     use vortex_dtype::Nullability;
     use vortex_mask::Mask;
 
-    use crate::array::{BoolArray, PrimitiveArray};
+    use crate::array::Array;
+    use crate::arrays::{BoolArray, PrimitiveArray};
     use crate::validity::Validity;
-    use crate::IntoArray;
+    use crate::{ArrayRef, IntoArray};
 
     #[rstest]
     #[case(Validity::AllValid, 5, &[2, 4], Validity::AllValid, Validity::AllValid)]
@@ -487,14 +520,17 @@ mod tests {
     ) {
         let indices =
             PrimitiveArray::new(Buffer::copy_from(positions), Validity::NonNullable).into_array();
-        assert_eq!(validity.patch(len, &indices, patches).unwrap(), expected);
+        assert_eq!(
+            validity.patch(len, 0, &indices, &patches).unwrap(),
+            expected
+        );
     }
 
     #[test]
     #[should_panic]
     fn out_of_bounds_patch() {
         Validity::NonNullable
-            .patch(2, &buffer![4].into_array(), Validity::AllInvalid)
+            .patch(2, 0, &buffer![4].into_array(), &Validity::AllInvalid)
             .unwrap();
     }
 
@@ -509,4 +545,19 @@ mod tests {
     fn into_validity_nullable_array() {
         Validity::from_mask(Mask::from_iter(vec![true, false]), Nullability::NonNullable);
     }
+
+    #[rstest]
+    #[case(Validity::AllValid, PrimitiveArray::new(buffer![0, 1], Validity::from_iter(vec![true, false])).into_array(), Validity::from_iter(vec![true, false]))]
+    #[case(Validity::AllValid, buffer![0, 1].into_array(), Validity::AllValid)]
+    #[case(Validity::AllValid, PrimitiveArray::new(buffer![0, 1], Validity::AllInvalid).into_array(), Validity::AllInvalid)]
+    #[case(Validity::NonNullable, PrimitiveArray::new(buffer![0, 1], Validity::from_iter(vec![true, false])).into_array(), Validity::from_iter(vec![true, false]))]
+    #[case(Validity::NonNullable, buffer![0, 1].into_array(), Validity::NonNullable)]
+    #[case(Validity::NonNullable, PrimitiveArray::new(buffer![0, 1], Validity::AllInvalid).into_array(), Validity::AllInvalid)]
+    fn validity_take(
+        #[case] validity: Validity,
+        #[case] indices: ArrayRef,
+        #[case] expected: Validity,
+    ) {
+        assert_eq!(validity.take(&indices).unwrap(), expected);
+    }
 }
