@@ -8,18 +8,23 @@
 
 pub use dtype::*;
 pub use extension::*;
+pub use field::*;
+pub use field_mask::*;
 pub use half;
 pub use nullability::*;
 pub use ptype::*;
+pub use struct_::*;
 
 #[cfg(feature = "arbitrary")]
 mod arbitrary;
 mod dtype;
 mod extension;
-pub mod field;
+mod field;
+mod field_mask;
 mod nullability;
 mod ptype;
 mod serde;
+mod struct_;
 
 #[cfg(feature = "proto")]
 pub mod proto {
@@ -30,7 +35,6 @@ pub mod proto {
     pub use vortex_proto::dtype;
 }
 
-#[cfg(feature = "flatbuffers")]
 pub mod flatbuffers {
     //! Flatbuffer representations for DTypes
     //!
