@@ -1,7 +1,8 @@
 use std::fmt::{Display, Formatter};
+use std::ops::BitOr;
 
 /// Whether an instance of a DType can be `null or not
-#[derive(Debug, Clone, Copy, Default, PartialEq, Eq, Hash, Ord, PartialOrd)]
+#[derive(Debug, Clone, Copy, Default, PartialEq, Eq, Hash)]
 pub enum Nullability {
     /// Instances of this DType are guaranteed to be non-nullable
     #[default]
@@ -10,6 +11,29 @@ pub enum Nullability {
     Nullable,
 }
 
+impl Nullability {
+    /// A self-describing displayed form.
+    ///
+    /// The usual Display renders [Nullability::NonNullable] as the empty string.
+    pub fn verbose_display(&self) -> impl Display {
+        match self {
+            Nullability::NonNullable => "NonNullable",
+            Nullability::Nullable => "Nullable",
+        }
+    }
+}
+
+impl BitOr for Nullability {
+    type Output = Nullability;
+
+    fn bitor(self, rhs: Self) -> Self::Output {
+        match (self, rhs) {
+            (Self::NonNullable, Self::NonNullable) => Self::NonNullable,
+            _ => Self::Nullable,
+        }
+    }
+}
+
 impl From<bool> for Nullability {
     fn from(value: bool) -> Self {
         if value {
