@@ -7,12 +7,12 @@ use std::panic::RefUnwindSafe;
 
 use num_traits::bounds::UpperBounded;
 use num_traits::{FromPrimitive, Num, NumCast, ToPrimitive};
-use vortex_error::{vortex_err, VortexError, VortexResult};
+use vortex_error::{VortexError, VortexResult, vortex_err};
 
-use crate::half::f16;
-use crate::nullability::Nullability::NonNullable;
 use crate::DType;
 use crate::DType::*;
+use crate::half::f16;
+use crate::nullability::Nullability::NonNullable;
 
 /// Physical type enum, represents the in-memory physical layout but might represent a different logical type.
 #[derive(Debug, Clone, Copy, PartialEq, PartialOrd, Eq, Hash)]
@@ -64,8 +64,6 @@ pub trait NativePType:
     + Copy
     + Debug
     + Display
-    + PartialEq
-    + PartialOrd
     + Default
     + RefUnwindSafe
     + Num
@@ -85,6 +83,30 @@ pub trait NativePType:
     /// Compare another instance of this type to `self`, providing a total ordering
     fn total_compare(self, other: Self) -> Ordering;
 
+    /// Test whether self is less than or equal to the other
+    #[inline]
+    fn is_le(self, other: Self) -> bool {
+        self.total_compare(other).is_le()
+    }
+
+    /// Test whether self is less than the other
+    #[inline]
+    fn is_lt(self, other: Self) -> bool {
+        self.total_compare(other).is_lt()
+    }
+
+    /// Test whether self is greater than or equal to the other
+    #[inline]
+    fn is_ge(self, other: Self) -> bool {
+        self.total_compare(other).is_ge()
+    }
+
+    /// Test whether self is greater than the other
+    #[inline]
+    fn is_gt(self, other: Self) -> bool {
+        self.total_compare(other).is_gt()
+    }
+
     /// Whether another instance of this type (`other`) is bitwise equal to `self`
     fn is_eq(self, other: Self) -> bool;
 }
@@ -183,6 +205,30 @@ macro_rules! match_each_native_ptype {
             PType::F32 => __with_floating_point__! { f32 },
             PType::F64 => __with_floating_point__! { f64 },
         }
+    });
+    ($self:expr,
+     unsigned: | $_:tt $unsigned_enc:ident | { $($unsigned_body:tt)* }
+     signed: | $_1:tt $signed_enc:ident | { $($signed_body:tt)* }
+     floating: | $_2:tt $floating_point_enc:ident | { $($floating_point_body:tt)* }
+    ) => ({
+        macro_rules! __with_unsigned__ {( $_ $unsigned_enc:ident ) => ( { $($unsigned_body)* } )}
+        macro_rules! __with_signed__ {( $_ $signed_enc:ident ) => ( { $($signed_body)* } )}
+        macro_rules! __with_floating_point__ {( $_ $floating_point_enc:ident ) => ( { $($floating_point_body)* } )}
+        use $crate::PType;
+        use $crate::half::f16;
+        match $self {
+            PType::U8 => __with_unsigned__! { u8 },
+            PType::U16 => __with_unsigned__! { u16 },
+            PType::U32 => __with_unsigned__! { u32 },
+            PType::U64 => __with_unsigned__! { u64 },
+            PType::I8 => __with_signed__! { i8 },
+            PType::I16 => __with_signed__! { i16 },
+            PType::I32 => __with_signed__! { i32 },
+            PType::I64 => __with_signed__! { i64 },
+            PType::F16 => __with_floating_point__! { f16 },
+            PType::F32 => __with_floating_point__! { f32 },
+            PType::F64 => __with_floating_point__! { f64 },
+        }
     })
 }
 
@@ -206,6 +252,27 @@ macro_rules! match_each_integer_ptype {
     })
 }
 
+/// Macro to match over each integer PType, binding the corresponding native type (from
+/// `NativePType`) and the corresponding unsigned type (also a `NativePType`).
+#[macro_export]
+macro_rules! match_each_integer_ptype_with_unsigned_type {
+    ($self:expr, | $_:tt $enc:ident, $_2:tt $unsigned:ident | $($body:tt)*) => ({
+        macro_rules! __with__ {( $_ $enc:ident, $_2 $unsigned:ident ) => ( $($body)* )}
+        use $crate::PType;
+        match $self {
+            PType::I8 => __with__! { i8, u8 },
+            PType::I16 => __with__! { i16, u16 },
+            PType::I32 => __with__! { i32, u32 },
+            PType::I64 => __with__! { i64, u64 },
+            PType::U8 => __with__! { u8, u8 },
+            PType::U16 => __with__! { u16, u16 },
+            PType::U32 => __with__! { u32, u32 },
+            PType::U64 => __with__! { u64, u64 },
+            other => panic!("Unsupported ptype {other}")
+        }
+    })
+}
+
 /// Macro to match over each unsigned integer type, binding the corresponding native type (from `NativePType`)
 #[macro_export]
 macro_rules! match_each_unsigned_integer_ptype {
@@ -238,6 +305,30 @@ macro_rules! match_each_float_ptype {
     })
 }
 
+/// Macro to match over each SIMD capable `PType`, binding the corresponding native type (from `NativePType`)
+///
+/// Note: The match will panic in case of `PType::F16`.
+#[macro_export]
+macro_rules! match_each_native_simd_ptype {
+    ($self:expr, | $_:tt $enc:ident | $($body:tt)*) => ({
+        macro_rules! __with__ {( $_ $enc:ident ) => ( $($body)* )}
+        use $crate::PType;
+        match $self {
+            PType::I8 => __with__! { i8 },
+            PType::I16 => __with__! { i16 },
+            PType::I32 => __with__! { i32 },
+            PType::I64 => __with__! { i64 },
+            PType::U8 => __with__! { u8 },
+            PType::U16 => __with__! { u16 },
+            PType::U32 => __with__! { u32 },
+            PType::U64 => __with__! { u64 },
+            PType::F16 => panic!("f16 does not implement simd::SimdElement"),
+            PType::F32 => __with__! { f32 },
+            PType::F64 => __with__! { f64 },
+        }
+    })
+}
+
 impl PType {
     /// Returns `true` iff this PType is an unsigned integer type
     pub const fn is_unsigned_int(self) -> bool {
@@ -381,7 +472,9 @@ macro_rules! try_from_bytes {
 
         impl TryFromBytes for $T {
             fn try_from_le_bytes(bytes: &[u8]) -> VortexResult<Self> {
-                Ok(<$T>::from_le_bytes(bytes.try_into()?))
+                Ok(<$T>::from_le_bytes(bytes.try_into().map_err(|_| {
+                    vortex_err!("Failed to convert bytes into {}", stringify!($T))
+                })?))
             }
         }
     };
