@@ -7,7 +7,7 @@ mod serde;
 #[cfg(test)]
 #[cfg(feature = "serde")]
 mod test {
-    use serde_test::{assert_tokens, Token};
+    use serde_test::{Token, assert_tokens};
 
     use crate::{DType, Nullability, PType};
 
