@@ -1,14 +1,6 @@
 use std::fmt::Display;
 
-use vortex_dtype::{FieldName, FieldNames};
-
-pub struct DisplayFieldName<'a>(pub &'a FieldName);
-
-impl Display for DisplayFieldName<'_> {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        write!(f, "${}", self.0)
-    }
-}
+use vortex_dtype::FieldNames;
 
 pub struct DisplayFieldNames<'a>(pub &'a FieldNames);
 
@@ -16,9 +8,9 @@ impl Display for DisplayFieldNames<'_> {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         for (i, field) in self.0.iter().enumerate() {
             if i > 0 {
-                write!(f, ",")?;
+                write!(f, ", ")?;
             }
-            DisplayFieldName(field).fmt(f)?
+            field.fmt(f)?
         }
         Ok(())
     }
