@@ -3,9 +3,9 @@ use std::fmt::Display;
 use std::sync::Arc;
 
 use itertools::Itertools;
-use vortex_array::Array;
-use vortex_dtype::FieldNames;
-use vortex_error::{vortex_bail, vortex_err, VortexResult};
+use vortex_array::{Array, ArrayRef, ArrayVariants};
+use vortex_dtype::{DType, FieldNames};
+use vortex_error::{VortexResult, vortex_bail, vortex_err};
 
 use crate::field::DisplayFieldNames;
 use crate::{ExprRef, VortexExpr};
@@ -108,15 +108,15 @@ impl SelectField {
 impl Display for SelectField {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
-            SelectField::Include(fields) => write!(f, "+({})", DisplayFieldNames(fields)),
-            SelectField::Exclude(fields) => write!(f, "-({})", DisplayFieldNames(fields)),
+            SelectField::Include(fields) => write!(f, "{{{}}}", DisplayFieldNames(fields)),
+            SelectField::Exclude(fields) => write!(f, "~{{{}}}", DisplayFieldNames(fields)),
         }
     }
 }
 
 impl Display for Select {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        write!(f, "select {} {}", self.fields, self.child)
+        write!(f, "{}{}", self.child, self.fields)
     }
 }
 
@@ -125,10 +125,10 @@ impl VortexExpr for Select {
         self
     }
 
-    fn unchecked_evaluate(&self, batch: &Array) -> VortexResult<Array> {
+    fn unchecked_evaluate(&self, batch: &dyn Array) -> VortexResult<ArrayRef> {
         let batch = self.child.evaluate(batch)?;
         let st = batch
-            .as_struct_array()
+            .as_struct_typed()
             .ok_or_else(|| vortex_err!("Not a struct array"))?;
         match &self.fields {
             SelectField::Include(f) => st.project(f),
@@ -152,6 +152,29 @@ impl VortexExpr for Select {
         assert_eq!(children.len(), 1);
         Self::new_expr(self.fields.clone(), children[0].clone())
     }
+
+    fn return_dtype(&self, scope_dtype: &DType) -> VortexResult<DType> {
+        let child_dtype = self.child.return_dtype(scope_dtype)?;
+        let child_struct_dtype = child_dtype
+            .as_struct()
+            .ok_or_else(|| vortex_err!("Select child not a struct dtype"))?;
+
+        let projected = match &self.fields {
+            SelectField::Include(fields) => child_struct_dtype.project(fields)?,
+            SelectField::Exclude(fields) => child_struct_dtype
+                .names()
+                .iter()
+                .cloned()
+                .zip_eq(child_struct_dtype.fields())
+                .filter(|(name, _)| !fields.contains(name))
+                .collect(),
+        };
+
+        Ok(DType::Struct(
+            Arc::new(projected),
+            child_dtype.nullability(),
+        ))
+    }
 }
 
 impl PartialEq for Select {
@@ -164,10 +187,10 @@ impl PartialEq for Select {
 mod tests {
     use std::sync::Arc;
 
-    use vortex_array::array::StructArray;
-    use vortex_array::IntoArray;
+    use vortex_array::arrays::StructArray;
+    use vortex_array::{ArrayVariants, IntoArray};
     use vortex_buffer::buffer;
-    use vortex_dtype::{DType, Field, FieldName, Nullability};
+    use vortex_dtype::{DType, FieldName, Nullability};
 
     use crate::{ident, select, select_exclude, test_harness};
 
@@ -183,8 +206,8 @@ mod tests {
     pub fn include_columns() {
         let st = test_array();
         let select = select(vec![FieldName::from("a")], ident());
-        let selected = select.evaluate(st.as_ref()).unwrap();
-        let selected_names = selected.as_struct_array().unwrap().names().clone();
+        let selected = select.evaluate(&st).unwrap();
+        let selected_names = selected.as_struct_typed().unwrap().names().clone();
         assert_eq!(selected_names.as_ref(), &["a".into()]);
     }
 
@@ -192,8 +215,8 @@ mod tests {
     pub fn exclude_columns() {
         let st = test_array();
         let select = select_exclude(vec![FieldName::from("a")], ident());
-        let selected = select.evaluate(st.as_ref()).unwrap();
-        let selected_names = selected.as_struct_array().unwrap().names().clone();
+        let selected = select.evaluate(&st).unwrap();
+        let selected_names = selected.as_struct_typed().unwrap().names().clone();
         assert_eq!(selected_names.as_ref(), &["b".into()]);
     }
 
@@ -203,13 +226,7 @@ mod tests {
 
         let select_expr = select(vec![FieldName::from("a")], ident());
         let expected_dtype = DType::Struct(
-            Arc::new(
-                dtype
-                    .as_struct()
-                    .unwrap()
-                    .project(&[Field::from("a")])
-                    .unwrap(),
-            ),
+            Arc::new(dtype.as_struct().unwrap().project(&["a".into()]).unwrap()),
             Nullability::NonNullable,
         );
         assert_eq!(select_expr.return_dtype(&dtype).unwrap(), expected_dtype);
@@ -239,7 +256,7 @@ mod tests {
                     dtype
                         .as_struct()
                         .unwrap()
-                        .project(&[Field::from("a"), Field::from("bool1"), Field::from("bool2")])
+                        .project(&["a".into(), "bool1".into(), "bool2".into()])
                         .unwrap()
                 ),
                 Nullability::NonNullable
