@@ -1,13 +1,13 @@
 use flatbuffers::Follow;
-use vortex_error::{vortex_err, VortexError};
-use vortex_flatbuffers::{footer as fb, FlatBufferRoot, ReadFlatBuffer, WriteFlatBuffer};
+use vortex_error::{VortexError, vortex_err};
+use vortex_flatbuffers::{FlatBufferRoot, ReadFlatBuffer, WriteFlatBuffer, footer as fb};
 
 use crate::footer::segment::Segment;
 
 /// Captures the layout information of a Vortex file.
 pub(crate) struct Postscript {
-    pub(crate) dtype: Segment,
-    pub(crate) file_layout: Segment,
+    pub(crate) dtype: Option<Segment>,
+    pub(crate) footer: Segment,
 }
 
 impl FlatBufferRoot for Postscript {}
@@ -19,13 +19,13 @@ impl WriteFlatBuffer for Postscript {
         &self,
         fbb: &mut flatbuffers::FlatBufferBuilder<'fb>,
     ) -> flatbuffers::WIPOffset<Self::Target<'fb>> {
-        let dtype = fb::Segment::from(&self.dtype);
-        let file_layout = fb::Segment::from(&self.file_layout);
+        let dtype = self.dtype.as_ref().map(fb::Segment::from);
+        let footer = fb::Segment::from(&self.footer);
         fb::Postscript::create(
             fbb,
             &fb::PostscriptArgs {
-                dtype: Some(&dtype),
-                file_layout: Some(&file_layout),
+                dtype: dtype.as_ref(),
+                footer: Some(&footer),
             },
         )
     }
@@ -39,13 +39,10 @@ impl ReadFlatBuffer for Postscript {
         fb: &<Self::Source<'buf> as Follow<'buf>>::Inner,
     ) -> Result<Self, Self::Error> {
         Ok(Self {
-            dtype: Segment::try_from(
-                fb.dtype()
-                    .ok_or_else(|| vortex_err!("Postscript missing dtype segment"))?,
-            )?,
-            file_layout: Segment::try_from(
-                fb.file_layout()
-                    .ok_or_else(|| vortex_err!("Postscript missing file_layout segment"))?,
+            dtype: fb.dtype().map(Segment::try_from).transpose()?,
+            footer: Segment::try_from(
+                fb.footer()
+                    .ok_or_else(|| vortex_err!("Postscript missing footer segment"))?,
             )?,
         })
     }
