@@ -4,25 +4,27 @@ use std::pin::pin;
 use std::sync::Arc;
 
 use bytes::Bytes;
-use futures::StreamExt;
-use futures_executor::block_on;
-use futures_util::TryStreamExt;
+use futures::{StreamExt, TryStreamExt, pin_mut};
 use itertools::Itertools;
 use vortex_array::accessor::ArrayAccessor;
-use vortex_array::array::{ChunkedArray, ListArray, PrimitiveArray, StructArray, VarBinArray};
+use vortex_array::arrays::{
+    ChunkedArray, ListArray, PrimitiveArray, StructArray, VarBinArray, VarBinViewArray,
+};
 use vortex_array::compute::scalar_at;
-use vortex_array::stream::ArrayStreamExt;
+use vortex_array::stream::ArrayStreamArrayExt;
 use vortex_array::validity::Validity;
 use vortex_array::variants::{PrimitiveArrayTrait, StructArrayTrait};
-use vortex_array::{Array, Context, ContextRef, IntoArray, IntoArrayVariant};
-use vortex_buffer::{buffer, Buffer, ByteBufferMut};
+use vortex_array::{Array, ArrayVariants, IntoArray, ToCanonical};
+use vortex_buffer::{Buffer, ByteBufferMut, buffer};
 use vortex_dtype::PType::I32;
 use vortex_dtype::{DType, Nullability, PType, StructDType};
-use vortex_error::{vortex_panic, VortexExpect, VortexResult};
+use vortex_error::{VortexResult, vortex_panic};
 use vortex_expr::{and, eq, get_item, gt, gt_eq, ident, lit, lt, lt_eq, or, select};
 
-use crate::io::IoDriver;
-use crate::{Scan, VortexFile, VortexOpenOptions, VortexWriteOptions, V1_FOOTER_FBS_SIZE, VERSION};
+use crate::{
+    InMemoryVortexFile, V1_FOOTER_FBS_SIZE, VERSION, VortexFile, VortexOpenOptions,
+    VortexWriteOptions,
+};
 
 #[test]
 fn test_eof_values() {
@@ -49,26 +51,26 @@ async fn test_read_simple() {
 
     let st = StructArray::from_fields(&[("strings", strings), ("numbers", numbers)]).unwrap();
     let buf = VortexWriteOptions::default()
-        .write(ByteBufferMut::empty(), st.into_array().into_array_stream())
+        .write(ByteBufferMut::empty(), st.to_array_stream())
         .await
         .unwrap();
 
-    let mut stream = pin!(VortexOpenOptions::new(Arc::new(Context::default()))
-        .open(buf.freeze())
+    let stream = VortexOpenOptions::in_memory(buf)
+        .open()
         .await
         .unwrap()
-        .scan(Scan::all())
-        .unwrap());
-    let mut batch_count = 0;
+        .scan()
+        .into_array_stream()
+        .unwrap();
+    pin_mut!(stream);
+
     let mut row_count = 0;
 
     while let Some(array) = stream.next().await {
         let array = array.unwrap();
-        batch_count += 1;
         row_count += array.len();
     }
 
-    assert_eq!(batch_count, 2);
     assert_eq!(row_count, 8);
 }
 
@@ -88,12 +90,12 @@ async fn test_read_simple_with_spawn() {
     .into_array();
 
     let lists = ChunkedArray::from_iter([
-        ListArray::from_iter_slow(
+        ListArray::from_iter_slow::<i16, _>(
             vec![vec![11, 12], vec![21, 22], vec![31, 32], vec![41, 42]],
             Arc::new(I32.into()),
         )
         .unwrap(),
-        ListArray::from_iter_slow(
+        ListArray::from_iter_slow::<i8, _>(
             vec![vec![51, 52], vec![61, 62], vec![71, 72], vec![81, 82]],
             Arc::new(I32.into()),
         )
@@ -106,7 +108,7 @@ async fn test_read_simple_with_spawn() {
             .unwrap();
 
     let buf = VortexWriteOptions::default()
-        .write(ByteBufferMut::empty(), st.into_array().into_array_stream())
+        .write(ByteBufferMut::empty(), st.to_array_stream())
         .await
         .unwrap();
 
@@ -135,18 +137,15 @@ async fn test_read_projection() {
     let st = StructArray::from_fields(&[("strings", strings), ("numbers", numbers)]).unwrap();
 
     let buf = VortexWriteOptions::default()
-        .write(ByteBufferMut::empty(), st.into_array().into_array_stream())
+        .write(ByteBufferMut::empty(), st.to_array_stream())
         .await
         .unwrap();
 
-    let file = VortexOpenOptions::new(Arc::new(Context::default()))
-        .open(buf.freeze())
-        .await
-        .unwrap();
+    let file = VortexOpenOptions::in_memory(buf).open().await.unwrap();
     let array = file
-        .scan(Scan::new(select(["strings".into()], ident())))
-        .unwrap()
-        .into_array_data()
+        .scan()
+        .with_projection(select(["strings".into()], ident()))
+        .into_array()
         .await
         .unwrap();
 
@@ -162,11 +161,11 @@ async fn test_read_projection() {
     );
 
     let actual = array
-        .into_struct()
+        .as_struct_typed()
         .unwrap()
         .maybe_null_field_by_idx(0)
         .unwrap()
-        .into_varbinview()
+        .to_varbinview()
         .unwrap()
         .with_iterator(|x| {
             x.map(|x| unsafe { String::from_utf8_unchecked(x.unwrap().to_vec()) })
@@ -176,9 +175,9 @@ async fn test_read_projection() {
     assert_eq!(actual, strings_expected);
 
     let array = file
-        .scan(Scan::new(select(["numbers".into()], ident())))
-        .unwrap()
-        .into_array_data()
+        .scan()
+        .with_projection(select(["numbers".into()], ident()))
+        .into_array()
         .await
         .unwrap();
 
@@ -194,11 +193,11 @@ async fn test_read_projection() {
     );
 
     let primitive_array = array
-        .into_struct()
+        .as_struct_typed()
         .unwrap()
         .maybe_null_field_by_idx(0)
         .unwrap()
-        .into_primitive()
+        .to_primitive()
         .unwrap();
     let actual = primitive_array.as_slice::<u32>();
     assert_eq!(actual, numbers_expected);
@@ -221,39 +220,39 @@ async fn unequal_batches() {
 
     let st = StructArray::from_fields(&[("strings", strings), ("numbers", numbers)]).unwrap();
     let buf = VortexWriteOptions::default()
-        .write(ByteBufferMut::empty(), st.into_array().into_array_stream())
+        .write(ByteBufferMut::empty(), st.to_array_stream())
         .await
         .unwrap();
 
-    let mut stream = pin!(VortexOpenOptions::new(Arc::new(Context::default()))
-        .open(buf.freeze())
-        .await
-        .unwrap()
-        .scan(Scan::all())
-        .unwrap());
+    let mut stream = pin!(
+        VortexOpenOptions::in_memory(buf)
+            .open()
+            .await
+            .unwrap()
+            .scan()
+            .into_array_stream()
+            .unwrap()
+    );
 
-    let mut batch_count = 0;
     let mut item_count = 0;
 
     while let Some(array) = stream.next().await {
         let array = array.unwrap();
         item_count += array.len();
-        batch_count += 1;
 
         let numbers = array
-            .as_struct_array()
+            .as_struct_typed()
             .unwrap()
             .maybe_null_field_by_name("numbers");
 
-        if let Some(numbers) = numbers {
-            let numbers = numbers.into_primitive().unwrap();
+        if let Ok(numbers) = numbers {
+            let numbers = numbers.to_primitive().unwrap();
             assert_eq!(numbers.ptype(), PType::U32);
         } else {
             vortex_panic!("Expected column doesn't exist")
         }
     }
     assert_eq!(item_count, 10);
-    assert_eq!(batch_count, 3);
 }
 
 #[tokio::test]
@@ -284,16 +283,19 @@ async fn write_chunked() {
         .unwrap()
         .into_array();
     let buf = VortexWriteOptions::default()
-        .write(ByteBufferMut::empty(), chunked_st.into_array_stream())
+        .write(ByteBufferMut::empty(), chunked_st.to_array_stream())
         .await
         .unwrap();
 
-    let mut stream = pin!(VortexOpenOptions::new(Arc::new(Context::default()))
-        .open(buf.freeze())
-        .await
-        .unwrap()
-        .scan(Scan::all())
-        .unwrap());
+    let mut stream = pin!(
+        VortexOpenOptions::in_memory(buf)
+            .open()
+            .await
+            .unwrap()
+            .scan()
+            .into_array_stream()
+            .unwrap()
+    );
     let mut array_len: usize = 0;
     while let Some(array) = stream.next().await {
         array_len += array.unwrap().len();
@@ -320,15 +322,17 @@ async fn filter_string() {
     .unwrap()
     .into_array();
     let buf = VortexWriteOptions::default()
-        .write(ByteBufferMut::empty(), st.into_array_stream())
+        .write(ByteBufferMut::empty(), st.to_array_stream())
         .await
         .unwrap();
 
-    let result = VortexOpenOptions::new(Arc::new(Context::default()))
-        .open(buf.freeze())
+    let result = VortexOpenOptions::in_memory(buf)
+        .open()
         .await
         .unwrap()
-        .scan(Scan::all().with_filter(eq(get_item("name", ident()), lit("Joseph"))))
+        .scan()
+        .with_filter(eq(get_item("name", ident()), lit("Joseph")))
+        .into_array_stream()
         .unwrap()
         .try_collect::<Vec<_>>()
         .await
@@ -336,13 +340,13 @@ async fn filter_string() {
 
     assert_eq!(result.len(), 1);
     let names = result[0]
-        .as_struct_array()
+        .as_struct_typed()
         .unwrap()
         .maybe_null_field_by_idx(0)
         .unwrap();
     assert_eq!(
         names
-            .into_varbinview()
+            .to_varbinview()
             .unwrap()
             .with_iterator(|iter| iter
                 .flatten()
@@ -352,11 +356,11 @@ async fn filter_string() {
         vec!["Joseph".to_string()]
     );
     let ages = result[0]
-        .as_struct_array()
+        .as_struct_typed()
         .unwrap()
         .maybe_null_field_by_idx(1)
         .unwrap();
-    assert_eq!(ages.into_primitive().unwrap().as_slice::<i32>(), vec![25]);
+    assert_eq!(ages.to_primitive().unwrap().as_slice::<i32>(), vec![25]);
 }
 
 #[tokio::test]
@@ -377,21 +381,23 @@ async fn filter_or() {
     .into_array();
 
     let buf = VortexWriteOptions::default()
-        .write(ByteBufferMut::empty(), st.into_array_stream())
+        .write(ByteBufferMut::empty(), st.to_array_stream())
         .await
         .unwrap();
 
-    let result = VortexOpenOptions::new(Arc::new(Context::default()))
-        .open(buf.freeze())
+    let result = VortexOpenOptions::in_memory(buf)
+        .open()
         .await
         .unwrap()
-        .scan(Scan::all().with_filter(or(
+        .scan()
+        .with_filter(or(
             eq(get_item("name", ident()), lit("Angela")),
             and(
                 gt_eq(get_item("age", ident()), lit(20)),
                 lt_eq(get_item("age", ident()), lit(30)),
             ),
-        )))
+        ))
+        .into_array_stream()
         .unwrap()
         .try_collect::<Vec<_>>()
         .await
@@ -399,13 +405,13 @@ async fn filter_or() {
 
     assert_eq!(result.len(), 1);
     let names = result[0]
-        .as_struct_array()
+        .as_struct_typed()
         .unwrap()
         .maybe_null_field_by_idx(0)
         .unwrap();
     assert_eq!(
         names
-            .into_varbinview()
+            .to_varbinview()
             .unwrap()
             .with_iterator(|iter| iter
                 .flatten()
@@ -415,12 +421,12 @@ async fn filter_or() {
         vec!["Joseph".to_string(), "Angela".to_string()]
     );
     let ages = result[0]
-        .as_struct_array()
+        .as_struct_typed()
         .unwrap()
         .maybe_null_field_by_idx(1)
         .unwrap();
     assert_eq!(
-        ages.into_primitive()
+        ages.to_primitive()
             .unwrap()
             .with_iterator(|iter| iter.map(|x| x.cloned()).collect::<Vec<_>>())
             .unwrap(),
@@ -446,18 +452,20 @@ async fn filter_and() {
     .into_array();
 
     let buf = VortexWriteOptions::default()
-        .write(ByteBufferMut::empty(), st.into_array_stream())
+        .write(ByteBufferMut::empty(), st.to_array_stream())
         .await
         .unwrap();
 
-    let result = VortexOpenOptions::new(Arc::new(Context::default()))
-        .open(buf.freeze())
+    let result = VortexOpenOptions::in_memory(buf)
+        .open()
         .await
         .unwrap()
-        .scan(Scan::all().with_filter(and(
+        .scan()
+        .with_filter(and(
             gt(get_item("age", ident()), lit(21)),
             lt_eq(get_item("age", ident()), lit(33)),
-        )))
+        ))
+        .into_array_stream()
         .unwrap()
         .try_collect::<Vec<_>>()
         .await
@@ -465,13 +473,13 @@ async fn filter_and() {
 
     assert_eq!(result.len(), 1);
     let names = result[0]
-        .as_struct_array()
+        .as_struct_typed()
         .unwrap()
         .maybe_null_field_by_idx(0)
         .unwrap();
     assert_eq!(
         names
-            .into_varbinview()
+            .to_varbinview()
             .unwrap()
             .with_iterator(|iter| iter
                 .map(|s| s.map(|st| unsafe { String::from_utf8_unchecked(st.to_vec()) }))
@@ -480,14 +488,11 @@ async fn filter_and() {
         vec![Some("Joseph".to_string()), None]
     );
     let ages = result[0]
-        .as_struct_array()
+        .as_struct_typed()
         .unwrap()
         .maybe_null_field_by_idx(1)
         .unwrap();
-    assert_eq!(
-        ages.into_primitive().unwrap().as_slice::<i32>(),
-        vec![25, 31]
-    );
+    assert_eq!(ages.to_primitive().unwrap().as_slice::<i32>(), vec![25, 31]);
 }
 
 #[tokio::test]
@@ -496,33 +501,32 @@ async fn test_with_indices_simple() {
     let expected_numbers_split: Vec<Buffer<i16>> = (0..5).map(|_| (0_i16..100).collect()).collect();
     let expected_array = StructArray::from_fields(&[(
         "numbers",
-        ChunkedArray::from_iter(expected_numbers_split.iter().cloned().map(Array::from))
-            .into_array(),
+        ChunkedArray::from_iter(
+            expected_numbers_split
+                .iter()
+                .cloned()
+                .map(IntoArray::into_array),
+        )
+        .into_array(),
     )])
     .unwrap();
     let expected_numbers: Vec<i16> = expected_numbers_split.into_iter().flatten().collect();
 
     let buf = VortexWriteOptions::default()
-        .write(
-            ByteBufferMut::empty(),
-            expected_array.into_array().into_array_stream(),
-        )
+        .write(ByteBufferMut::empty(), expected_array.to_array_stream())
         .await
         .unwrap();
 
-    let file = VortexOpenOptions::new(Arc::new(Context::default()))
-        .open(buf.freeze())
-        .await
-        .unwrap();
+    let file = VortexOpenOptions::in_memory(buf).open().await.unwrap();
 
     // test no indices
     let actual_kept_array = file
-        .scan(Scan::all().with_row_indices(Buffer::<u64>::empty()))
-        .unwrap()
-        .into_array_data()
+        .scan()
+        .with_row_indices(Buffer::<u64>::empty())
+        .into_array()
         .await
         .unwrap()
-        .into_struct()
+        .to_struct()
         .unwrap();
 
     assert_eq!(actual_kept_array.len(), 0);
@@ -531,17 +535,17 @@ async fn test_with_indices_simple() {
     let kept_indices = [0_u64, 3, 99, 100, 101, 399, 400, 401, 499];
 
     let actual_kept_array = file
-        .scan(Scan::all().with_row_indices(Buffer::from_iter(kept_indices)))
-        .unwrap()
-        .into_array_data()
+        .scan()
+        .with_row_indices(Buffer::from_iter(kept_indices))
+        .into_array()
         .await
         .unwrap()
-        .into_struct()
+        .to_struct()
         .unwrap();
     let actual_kept_numbers_array = actual_kept_array
         .maybe_null_field_by_idx(0)
         .unwrap()
-        .into_primitive()
+        .to_primitive()
         .unwrap();
 
     let expected_kept_numbers: Vec<i16> = kept_indices
@@ -554,17 +558,17 @@ async fn test_with_indices_simple() {
 
     // test all indices
     let actual_array = file
-        .scan(Scan::all().with_row_indices((0u64..500).collect::<Buffer<_>>()))
-        .unwrap()
-        .into_array_data()
+        .scan()
+        .with_row_indices((0u64..500).collect::<Buffer<_>>())
+        .into_array()
         .await
         .unwrap()
-        .into_struct()
+        .to_struct()
         .unwrap();
     let actual_numbers_array = actual_array
         .maybe_null_field_by_idx(0)
         .unwrap()
-        .into_primitive()
+        .to_primitive()
         .unwrap();
     let actual_numbers = actual_numbers_array.as_slice::<i16>();
 
@@ -590,31 +594,28 @@ async fn test_with_indices_on_two_columns() {
 
     let st = StructArray::from_fields(&[("strings", strings), ("numbers", numbers)]).unwrap();
     let buf = VortexWriteOptions::default()
-        .write(ByteBufferMut::empty(), st.into_array().into_array_stream())
+        .write(ByteBufferMut::empty(), st.to_array_stream())
         .await
         .unwrap();
 
-    let file = VortexOpenOptions::new(Arc::new(Context::default()))
-        .open(buf.freeze())
-        .await
-        .unwrap();
+    let file = VortexOpenOptions::in_memory(buf).open().await.unwrap();
 
     let kept_indices = [0_u64, 3, 7];
     let array = file
-        .scan(Scan::all().with_row_indices(Buffer::from_iter(kept_indices)))
-        .unwrap()
-        .into_array_data()
+        .scan()
+        .with_row_indices(Buffer::from_iter(kept_indices))
+        .into_array()
         .await
         .unwrap()
-        .into_struct()
+        .to_struct()
         .unwrap()
-        .into_struct()
+        .to_struct()
         .unwrap();
 
     let strings_actual = array
         .maybe_null_field_by_idx(0)
         .unwrap()
-        .into_varbinview()
+        .to_varbinview()
         .unwrap()
         .with_iterator(|x| {
             x.map(|x| unsafe { String::from_utf8_unchecked(x.unwrap().to_vec()) })
@@ -632,7 +633,7 @@ async fn test_with_indices_on_two_columns() {
     let numbers_actual_array = array
         .maybe_null_field_by_idx(1)
         .unwrap()
-        .into_primitive()
+        .to_primitive()
         .unwrap();
     let numbers_actual = numbers_actual_array.as_slice::<u32>();
     assert_eq!(
@@ -650,38 +651,32 @@ async fn test_with_indices_and_with_row_filter_simple() {
     let expected_numbers_split: Vec<Buffer<i16>> = (0..5).map(|_| (0_i16..100).collect()).collect();
     let expected_array = StructArray::from_fields(&[(
         "numbers",
-        ChunkedArray::from_iter(expected_numbers_split.iter().cloned().map(Array::from))
-            .into_array(),
+        ChunkedArray::from_iter(
+            expected_numbers_split
+                .iter()
+                .cloned()
+                .map(IntoArray::into_array),
+        )
+        .into_array(),
     )])
     .unwrap();
     let expected_numbers: Vec<i16> = expected_numbers_split.into_iter().flatten().collect();
 
     let buf = VortexWriteOptions::default()
-        .write(
-            ByteBufferMut::empty(),
-            expected_array.into_array().into_array_stream(),
-        )
+        .write(ByteBufferMut::empty(), expected_array.to_array_stream())
         .await
         .unwrap();
 
-    let file = VortexOpenOptions::new(Arc::new(Context::default()))
-        .open(buf.freeze())
-        .await
-        .unwrap();
+    let file = VortexOpenOptions::in_memory(buf).open().await.unwrap();
 
     let actual_kept_array = file
-        .scan(
-            Scan::all()
-                .with_filter(gt(get_item("numbers", ident()), lit(50_i16)))
-                .with_row_indices(Buffer::empty()),
-        )
-        .unwrap()
-        .into_array_data()
+        .scan()
+        .with_filter(gt(get_item("numbers", ident()), lit(50_i16)))
+        .with_row_indices(Buffer::empty())
+        .into_array()
         .await
         .unwrap()
-        .into_struct()
-        .unwrap()
-        .into_struct()
+        .to_struct()
         .unwrap();
 
     assert_eq!(actual_kept_array.len(), 0);
@@ -690,24 +685,19 @@ async fn test_with_indices_and_with_row_filter_simple() {
     let kept_indices = [0u64, 3, 99, 100, 101, 399, 400, 401, 499];
 
     let actual_kept_array = file
-        .scan(
-            Scan::all()
-                .with_filter(gt(get_item("numbers", ident()), lit(50_i16)))
-                .with_row_indices(Buffer::from_iter(kept_indices)),
-        )
-        .unwrap()
-        .into_array_data()
+        .scan()
+        .with_filter(gt(get_item("numbers", ident()), lit(50_i16)))
+        .with_row_indices(Buffer::from_iter(kept_indices))
+        .into_array()
         .await
         .unwrap()
-        .into_struct()
-        .unwrap()
-        .into_struct()
+        .to_struct()
         .unwrap();
 
     let actual_kept_numbers_array = actual_kept_array
         .maybe_null_field_by_idx(0)
         .unwrap()
-        .into_primitive()
+        .to_primitive()
         .unwrap();
 
     let expected_kept_numbers: Buffer<i16> = kept_indices
@@ -721,24 +711,19 @@ async fn test_with_indices_and_with_row_filter_simple() {
 
     // test all indices
     let actual_array = file
-        .scan(
-            Scan::all()
-                .with_filter(gt(get_item("numbers", ident()), lit(50_i16)))
-                .with_row_indices((0..500).collect::<Buffer<_>>()),
-        )
-        .unwrap()
-        .into_array_data()
+        .scan()
+        .with_filter(gt(get_item("numbers", ident()), lit(50_i16)))
+        .with_row_indices((0..500).collect::<Buffer<_>>())
+        .into_array()
         .await
         .unwrap()
-        .into_struct()
-        .unwrap()
-        .into_struct()
+        .to_struct()
         .unwrap();
 
     let actual_numbers_array = actual_array
         .maybe_null_field_by_idx(0)
         .unwrap()
-        .into_primitive()
+        .to_primitive()
         .unwrap();
     let actual_numbers = actual_numbers_array.as_slice::<i16>();
 
@@ -755,19 +740,19 @@ async fn test_with_indices_and_with_row_filter_simple() {
 #[tokio::test]
 #[cfg_attr(miri, ignore)]
 async fn filter_string_chunked() {
-    let name_chunk1 = Array::from_iter(vec![
-        Some("Joseph".to_owned()),
-        Some("James".to_owned()),
-        Some("Angela".to_owned()),
-    ]);
-    let age_chunk1 = Array::from_iter(vec![Some(25_i32), Some(31), None]);
-    let name_chunk2 = Array::from_iter(vec![
+    let name_chunk1 =
+        VarBinViewArray::from_iter_nullable_str([Some("Joseph"), Some("James"), Some("Angela")])
+            .into_array();
+    let age_chunk1 = PrimitiveArray::from_option_iter([Some(25_i32), Some(31), None]).into_array();
+    let name_chunk2 = VarBinViewArray::from_iter_nullable_str([
         Some("Pharrell".to_owned()),
         Some("Khalil".to_owned()),
         Some("Mikhail".to_owned()),
         None,
-    ]);
-    let age_chunk2 = Array::from_iter(vec![Some(57_i32), Some(18), None, Some(32)]);
+    ])
+    .into_array();
+    let age_chunk2 =
+        PrimitiveArray::from_option_iter([Some(57_i32), Some(18), None, Some(32)]).into_array();
 
     let chunk1 = StructArray::from_fields(&[("name", name_chunk1), ("age", age_chunk1)])
         .unwrap()
@@ -782,29 +767,26 @@ async fn filter_string_chunked() {
         .into_array();
 
     let buf = VortexWriteOptions::default()
-        .write(ByteBufferMut::empty(), array.into_array_stream())
+        .write(ByteBufferMut::empty(), array.to_array_stream())
         .await
         .unwrap();
 
-    let file = VortexOpenOptions::new(Arc::new(Context::default()))
-        .open(buf.freeze())
-        .await
-        .unwrap();
+    let file = VortexOpenOptions::in_memory(buf).open().await.unwrap();
 
     let actual_array = file
-        .scan(Scan::all().with_filter(eq(get_item("name", ident()), lit("Joseph"))))
-        .unwrap()
-        .into_array_data()
+        .scan()
+        .with_filter(eq(get_item("name", ident()), lit("Joseph")))
+        .into_array()
         .await
         .unwrap()
-        .into_struct()
+        .to_struct()
         .unwrap();
 
     assert_eq!(actual_array.len(), 1);
     let names = actual_array.maybe_null_field_by_idx(0).unwrap();
     assert_eq!(
         names
-            .into_varbinview()
+            .to_varbinview()
             .unwrap()
             .with_iterator(|iter| iter
                 .flatten()
@@ -814,38 +796,46 @@ async fn filter_string_chunked() {
         vec!["Joseph".to_string()]
     );
     let ages = actual_array.maybe_null_field_by_idx(1).unwrap();
-    assert_eq!(ages.into_primitive().unwrap().as_slice::<i32>(), vec![25]);
+    assert_eq!(ages.to_primitive().unwrap().as_slice::<i32>(), vec![25]);
 }
 
 #[tokio::test]
 #[cfg_attr(miri, ignore)]
 async fn test_pruning_with_or() {
-    let letter_chunk1 = Array::from_iter(vec![
+    let letter_chunk1 = VarBinViewArray::from_iter_nullable_str([
         Some("A".to_owned()),
         Some("B".to_owned()),
         Some("D".to_owned()),
-    ]);
-    let number_chunk1 = Array::from_iter(vec![Some(25_i32), Some(31), None]);
-    let letter_chunk2 = Array::from_iter(vec![
+    ])
+    .into_array();
+    let number_chunk1 =
+        PrimitiveArray::from_option_iter([Some(25_i32), Some(31), None]).into_array();
+    let letter_chunk2 = VarBinViewArray::from_iter_nullable_str([
         Some("G".to_owned()),
         Some("I".to_owned()),
         Some("J".to_owned()),
         None,
-    ]);
-    let number_chunk2 = Array::from_iter(vec![Some(4_i32), Some(18), None, Some(21)]);
-    let letter_chunk3 = Array::from_iter(vec![
+    ])
+    .into_array();
+    let number_chunk2 =
+        PrimitiveArray::from_option_iter([Some(4_i32), Some(18), None, Some(21)]).into_array();
+    let letter_chunk3 = VarBinViewArray::from_iter_nullable_str([
         Some("L".to_owned()),
         None,
         Some("O".to_owned()),
         Some("P".to_owned()),
-    ]);
-    let number_chunk3 = Array::from_iter(vec![Some(10_i32), Some(15), None, Some(22)]);
-    let letter_chunk4 = Array::from_iter(vec![
+    ])
+    .into_array();
+    let number_chunk3 =
+        PrimitiveArray::from_option_iter([Some(10_i32), Some(15), None, Some(22)]).into_array();
+    let letter_chunk4 = VarBinViewArray::from_iter_nullable_str([
         Some("X".to_owned()),
         Some("Y".to_owned()),
         Some("Z".to_owned()),
-    ]);
-    let number_chunk4 = Array::from_iter(vec![Some(66_i32), Some(77), Some(88)]);
+    ])
+    .into_array();
+    let number_chunk4 =
+        PrimitiveArray::from_option_iter([Some(66_i32), Some(77), Some(88)]).into_array();
 
     let chunk1 = StructArray::from_fields(&[("letter", letter_chunk1), ("number", number_chunk1)])
         .unwrap()
@@ -866,32 +856,29 @@ async fn test_pruning_with_or() {
         .into_array();
 
     let buf = VortexWriteOptions::default()
-        .write(ByteBufferMut::empty(), array.into_array_stream())
+        .write(ByteBufferMut::empty(), array.to_array_stream())
         .await
         .unwrap();
 
-    let file = VortexOpenOptions::new(Arc::new(Context::default()))
-        .open(buf.freeze())
-        .await
-        .unwrap();
+    let file = VortexOpenOptions::in_memory(buf).open().await.unwrap();
 
     let actual_array = file
-        .scan(Scan::all().with_filter(or(
+        .scan()
+        .with_filter(or(
             lt_eq(get_item("letter", ident()), lit("J")),
             lt(get_item("number", ident()), lit(25)),
-        )))
-        .unwrap()
-        .into_array_data()
+        ))
+        .into_array()
         .await
         .unwrap()
-        .into_struct()
+        .to_struct()
         .unwrap();
 
     assert_eq!(actual_array.len(), 10);
     let letters = actual_array.maybe_null_field_by_idx(0).unwrap();
     assert_eq!(
         letters
-            .into_varbinview()
+            .to_varbinview()
             .unwrap()
             .with_iterator(|iter| iter
                 .map(|opt| opt.map(|s| unsafe { String::from_utf8_unchecked(s.to_vec()) }))
@@ -954,26 +941,20 @@ async fn test_repeated_projection() {
     let buf = VortexWriteOptions::default()
         .write(
             ByteBufferMut::empty(),
-            single_column_array.into_array_stream(),
+            single_column_array.to_array_stream(),
         )
         .await
         .unwrap();
 
-    let file = VortexOpenOptions::new(Arc::new(Context::default()))
-        .open(buf.freeze())
-        .await
-        .unwrap();
+    let file = VortexOpenOptions::in_memory(buf).open().await.unwrap();
 
     let actual = file
-        .scan(Scan::new(select(
-            ["strings".into(), "strings".into()],
-            ident(),
-        )))
-        .unwrap()
-        .into_array_data()
+        .scan()
+        .with_projection(select(["strings".into(), "strings".into()], ident()))
+        .into_array()
         .await
         .unwrap()
-        .into_struct()
+        .to_struct()
         .unwrap();
 
     assert_eq!(
@@ -986,7 +967,7 @@ async fn test_repeated_projection() {
     );
 }
 
-fn chunked_file() -> VortexFile<impl IoDriver> {
+async fn chunked_file() -> VortexResult<VortexFile<InMemoryVortexFile>> {
     let array = ChunkedArray::from_iter([
         buffer![0, 1, 2].into_array(),
         buffer![3, 4, 5].into_array(),
@@ -994,36 +975,62 @@ fn chunked_file() -> VortexFile<impl IoDriver> {
     ])
     .into_array();
 
-    block_on(async {
-        let buffer: Bytes = VortexWriteOptions::default()
-            .write(vec![], array.into_array_stream())
-            .await?
-            .into();
-        VortexOpenOptions::new(ContextRef::default())
-            .open(buffer)
-            .await
-    })
-    .vortex_expect("Failed to create test file")
+    let buffer: Bytes = VortexWriteOptions::default()
+        .write(vec![], array.to_array_stream())
+        .await?
+        .into();
+    VortexOpenOptions::in_memory(buffer).open().await
 }
 
-#[test]
-fn basic_file_roundtrip() -> VortexResult<()> {
-    let vxf = chunked_file();
-    let result = block_on(vxf.scan(Scan::all())?.into_array_data())?.into_primitive()?;
+#[tokio::test]
+async fn basic_file_roundtrip() -> VortexResult<()> {
+    let vxf = chunked_file().await?;
+    let result = vxf.scan().into_array().await?.to_primitive()?;
 
     assert_eq!(result.as_slice::<i32>(), &[0, 1, 2, 3, 4, 5, 6, 7, 8]);
 
     Ok(())
 }
 
-#[test]
-fn file_take() -> VortexResult<()> {
-    let vxf = chunked_file();
-    let result = block_on(
-        vxf.scan(Scan::all().with_row_indices(buffer![0, 1, 8]))?
-            .into_array_data(),
-    )?
-    .into_primitive()?;
+#[tokio::test]
+async fn file_excluding_dtype() -> VortexResult<()> {
+    let array = ChunkedArray::from_iter([
+        buffer![0, 1, 2].into_array(),
+        buffer![3, 4, 5].into_array(),
+        buffer![6, 7, 8].into_array(),
+    ])
+    .into_array();
+    let dtype = array.dtype().clone();
+
+    let buffer: Bytes = VortexWriteOptions::default()
+        .exclude_dtype()
+        .write(vec![], array.to_array_stream())
+        .await?
+        .into();
+
+    // Fail to open without DType.
+    let vxf = VortexOpenOptions::in_memory(buffer.clone()).open().await;
+    assert!(vxf.is_err(), "Opening without DType should fail");
+
+    let vxf = VortexOpenOptions::in_memory(buffer)
+        .with_dtype(dtype.clone())
+        .open()
+        .await?;
+    assert_eq!(vxf.dtype(), &dtype);
+    assert_eq!(vxf.row_count(), 9);
+
+    Ok(())
+}
+
+#[tokio::test]
+async fn file_take() -> VortexResult<()> {
+    let vxf = chunked_file().await?;
+    let result = vxf
+        .scan()
+        .with_row_indices(buffer![0, 1, 8])
+        .into_array()
+        .await?
+        .to_primitive()?;
 
     assert_eq!(result.as_slice::<i32>(), &[0, 1, 8]);
 
