@@ -1,12 +1,11 @@
 mod eval_expr;
-mod eval_stats;
 mod reader;
 pub mod writer;
 
 use std::collections::BTreeSet;
 use std::sync::Arc;
 
-use vortex_array::ContextRef;
+use vortex_array::ArrayContext;
 use vortex_dtype::FieldMask;
 use vortex_error::VortexResult;
 
@@ -14,7 +13,7 @@ use crate::layouts::flat::reader::FlatReader;
 use crate::reader::{LayoutReader, LayoutReaderExt};
 use crate::segments::AsyncSegmentReader;
 use crate::vtable::LayoutVTable;
-use crate::{Layout, LayoutId, FLAT_LAYOUT_ID};
+use crate::{FLAT_LAYOUT_ID, Layout, LayoutId};
 
 #[derive(Debug)]
 pub struct FlatLayout;
@@ -27,10 +26,10 @@ impl LayoutVTable for FlatLayout {
     fn reader(
         &self,
         layout: Layout,
-        ctx: ContextRef,
-        segments: Arc<dyn AsyncSegmentReader>,
+        ctx: ArrayContext,
+        segment_reader: Arc<dyn AsyncSegmentReader>,
     ) -> VortexResult<Arc<dyn LayoutReader>> {
-        Ok(FlatReader::try_new(layout, ctx, segments)?.into_arc())
+        Ok(FlatReader::try_new(layout, ctx, segment_reader)?.into_arc())
     }
 
     fn register_splits(
