@@ -1,9 +1,9 @@
 use std::sync::Arc;
 
 use arrow_array::*;
-use vortex_datetime_dtype::{is_temporal_ext_type, TemporalMetadata, TimeUnit};
+use vortex_datetime_dtype::{TemporalMetadata, TimeUnit, is_temporal_ext_type};
 use vortex_dtype::{DType, PType};
-use vortex_error::{vortex_bail, vortex_err, VortexError};
+use vortex_error::{VortexError, vortex_bail, vortex_err};
 
 use crate::Scalar;
 
