@@ -2,7 +2,7 @@ use std::sync::Arc;
 
 use vortex_buffer::ByteBuffer;
 use vortex_dtype::{DType, Nullability};
-use vortex_error::{vortex_bail, vortex_err, VortexError, VortexExpect as _, VortexResult};
+use vortex_error::{VortexError, VortexExpect as _, VortexResult, vortex_bail, vortex_err};
 
 use crate::{InnerScalarValue, Scalar, ScalarValue};
 
@@ -14,16 +14,21 @@ pub struct BinaryScalar<'a> {
 
 impl PartialEq for BinaryScalar<'_> {
     fn eq(&self, other: &Self) -> bool {
-        self.dtype == other.dtype && self.value == other.value
+        self.dtype.eq_ignore_nullability(other.dtype) && self.value == other.value
     }
 }
 
 impl Eq for BinaryScalar<'_> {}
 
-/// Ord is not implemented since it's undefined for different nullability
 impl PartialOrd for BinaryScalar<'_> {
     fn partial_cmp(&self, other: &Self) -> Option<std::cmp::Ordering> {
-        self.value.partial_cmp(&other.value)
+        Some(self.value.cmp(&other.value))
+    }
+}
+
+impl Ord for BinaryScalar<'_> {
+    fn cmp(&self, other: &Self) -> std::cmp::Ordering {
+        self.value.cmp(&other.value)
     }
 }
 
@@ -51,6 +56,11 @@ impl<'a> BinaryScalar<'a> {
             ))),
         ))
     }
+
+    /// Returns whether its value is non-null and empty, otherwise `None`.
+    pub fn is_empty(&self) -> Option<bool> {
+        self.value.as_ref().map(|v| v.is_empty())
+    }
 }
 
 impl Scalar {
