@@ -34,7 +34,7 @@ pub use pvalue::*;
 pub use scalarvalue::*;
 pub use struct_::*;
 pub use utf8::*;
-use vortex_error::{vortex_bail, VortexExpect, VortexResult};
+use vortex_error::{VortexExpect, VortexResult, vortex_bail};
 
 /// A single logical item, composed of both a [`ScalarValue`] and a logical [`DType`].
 ///
@@ -61,9 +61,8 @@ impl Scalar {
         &self.dtype
     }
 
-    /// Only the scalar crate should access the ScalarValue directly.
     #[inline]
-    pub(crate) fn value(&self) -> &ScalarValue {
+    pub fn value(&self) -> &ScalarValue {
         &self.value
     }
 
@@ -109,7 +108,7 @@ impl Scalar {
         }
     }
 
-    pub fn cast_to_non_extension(&self, target: &DType) -> VortexResult<Self> {
+    fn cast_to_non_extension(&self, target: &DType) -> VortexResult<Self> {
         assert!(!matches!(target, DType::Extension(..)));
         if self.is_null() {
             if target.is_nullable() {
@@ -141,6 +140,32 @@ impl Scalar {
             value: self.value,
         }
     }
+
+    /// Size of the scalar in bytes, uncompressed.
+    pub fn nbytes(&self) -> usize {
+        match self.dtype() {
+            DType::Null => 0,
+            DType::Bool(_) => 1,
+            DType::Primitive(ptype, _) => ptype.byte_width(),
+            DType::Binary(_) | DType::Utf8(_) => self
+                .value()
+                .as_buffer()
+                .ok()
+                .flatten()
+                .map_or(0, |s| s.len()),
+            DType::Struct(_dtype, _) => self
+                .as_struct()
+                .fields()
+                .map(|fields| fields.into_iter().map(|f| f.nbytes()).sum::<usize>())
+                .unwrap_or_default(),
+            DType::List(_dtype, _) => self
+                .as_list()
+                .elements()
+                .map(|fields| fields.into_iter().map(|f| f.nbytes()).sum::<usize>())
+                .unwrap_or_default(),
+            DType::Extension(_ext_dtype) => self.as_extension().storage().nbytes(),
+        }
+    }
 }
 
 impl Scalar {
@@ -203,7 +228,7 @@ impl Scalar {
 
 impl PartialEq for Scalar {
     fn eq(&self, other: &Self) -> bool {
-        if self.dtype != other.dtype {
+        if !self.dtype.eq_ignore_nullability(&other.dtype) {
             return false;
         }
 
@@ -224,10 +249,9 @@ impl Eq for Scalar {}
 
 impl PartialOrd for Scalar {
     fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
-        if self.dtype() != other.dtype() {
+        if !self.dtype().eq_ignore_nullability(other.dtype()) {
             return None;
         }
-
         match self.dtype() {
             DType::Null => Some(Ordering::Equal),
             DType::Bool(_) => self.as_bool().partial_cmp(&other.as_bool()),
