@@ -3,9 +3,9 @@ use std::ops::Deref;
 use std::sync::Arc;
 
 use itertools::Itertools;
-use vortex_dtype::{DType, Field, FieldName, StructDType};
+use vortex_dtype::{DType, FieldName, StructDType};
 use vortex_error::{
-    vortex_bail, vortex_err, vortex_panic, VortexError, VortexExpect, VortexResult,
+    VortexError, VortexExpect, VortexResult, vortex_bail, vortex_err, vortex_panic,
 };
 
 use crate::{InnerScalarValue, Scalar, ScalarValue};
@@ -17,7 +17,7 @@ pub struct StructScalar<'a> {
 
 impl PartialEq for StructScalar<'_> {
     fn eq(&self, other: &Self) -> bool {
-        if self.dtype != other.dtype {
+        if !self.dtype.eq_ignore_nullability(other.dtype) {
             return false;
         }
         self.fields() == other.fields()
@@ -26,10 +26,10 @@ impl PartialEq for StructScalar<'_> {
 
 impl Eq for StructScalar<'_> {}
 
-/// Ord is not implemented since it's undefined for different DTypes
+/// Ord is not implemented since it's undefined for different field DTypes
 impl PartialOrd for StructScalar<'_> {
     fn partial_cmp(&self, other: &Self) -> Option<std::cmp::Ordering> {
-        if self.dtype() != other.dtype() {
+        if !self.dtype.eq_ignore_nullability(other.dtype) {
             return None;
         }
         self.fields().partial_cmp(&other.fields())
@@ -71,38 +71,39 @@ impl<'a> StructScalar<'a> {
         self.fields.is_none()
     }
 
-    pub fn field_by_idx(&self, idx: usize) -> Option<Scalar> {
-        let DType::Struct(st, nullability) = self.dtype() else {
+    pub fn field(&self, name: impl AsRef<str>) -> VortexResult<Scalar> {
+        let DType::Struct(st, _) = self.dtype() else {
             unreachable!()
         };
-
-        let field_dtype = st.field_dtype(idx).ok()?.with_nullability(*nullability);
-
-        Some(
-            self.fields
-                .and_then(|values| values.get(idx))
-                .map(|value| Scalar {
-                    dtype: field_dtype.clone(),
-                    value: value.clone(),
-                })
-                .unwrap_or_else(|| Scalar::null(field_dtype)),
-        )
+        let idx = st.find(name)?;
+        self.field_by_idx(idx)
     }
 
-    pub fn field_by_name(&self, name: &str) -> Option<Scalar> {
+    pub fn field_by_idx(&self, idx: usize) -> VortexResult<Scalar> {
+        let fields = self
+            .fields
+            .vortex_expect("Can't take field out of null struct scalar");
         let DType::Struct(st, _) = self.dtype() else {
             unreachable!()
         };
 
-        st.find_name(name).and_then(|idx| self.field_by_idx(idx))
+        Ok(Scalar {
+            dtype: st.field_by_index(idx)?,
+            value: fields[idx].clone(),
+        })
     }
 
+    /// Returns the fields of the struct scalar, or None if the scalar is null.
     pub fn fields(&self) -> Option<Vec<Scalar>> {
         let fields = self.fields?;
-
-        (0..fields.len())
-            .map(|index| self.field_by_idx(index))
-            .collect::<Option<Vec<_>>>()
+        Some(
+            (0..fields.len())
+                .map(|index| {
+                    self.field_by_idx(index)
+                        .vortex_expect("never out of bounds")
+                })
+                .collect::<Vec<_>>(),
+        )
     }
 
     pub(crate) fn field_values(&self) -> Option<&[ScalarValue]> {
@@ -117,11 +118,11 @@ impl<'a> StructScalar<'a> {
             unreachable!()
         };
 
-        if st.dtypes().len() != own_st.dtypes().len() {
+        if st.fields().len() != own_st.fields().len() {
             vortex_bail!(
                 "Cannot cast between structs with different number of fields: {} and {}",
-                own_st.dtypes().len(),
-                st.dtypes().len()
+                own_st.fields().len(),
+                st.fields().len()
             );
         }
 
@@ -131,10 +132,10 @@ impl<'a> StructScalar<'a> {
                 .enumerate()
                 .map(|(i, f)| {
                     Scalar {
-                        dtype: own_st.field_dtype(i)?,
+                        dtype: own_st.field_by_index(i)?,
                         value: f.clone(),
                     }
-                    .cast(&st.field_dtype(i)?)
+                    .cast(&st.field_by_index(i)?)
                     .map(|s| s.value)
                 })
                 .collect::<VortexResult<Vec<_>>>()?;
@@ -152,20 +153,14 @@ impl<'a> StructScalar<'a> {
             .dtype
             .as_struct()
             .ok_or_else(|| vortex_err!("Not a struct dtype"))?;
-        let projected_dtype = struct_dtype.project(
-            projection
-                .iter()
-                .map(|f| Field::Name(f.clone()))
-                .collect_vec()
-                .as_slice(),
-        )?;
+        let projected_dtype = struct_dtype.project(projection)?;
         let new_fields = if let Some(fs) = self.field_values() {
             ScalarValue(InnerScalarValue::List(
                 projection
                     .iter()
                     .map(|name| {
                         struct_dtype
-                            .find_name(name)
+                            .find(name)
                             .vortex_expect("DType has been successfully projected already")
                     })
                     .map(|i| fs[i].clone())
@@ -214,42 +209,31 @@ mod tests {
     fn setup_types() -> (DType, DType, DType) {
         let f0_dt = DType::Primitive(I32, Nullability::NonNullable);
         let f1_dt = DType::Utf8(Nullability::NonNullable);
-        let f0_dt_null = f0_dt.with_nullability(Nullability::Nullable);
-        let f1_dt_null = f1_dt.with_nullability(Nullability::Nullable);
 
         let dtype = DType::Struct(
             Arc::new(StructDType::new(
                 vec!["a".into(), "b".into()].into(),
-                vec![f0_dt, f1_dt],
+                vec![f0_dt.clone(), f1_dt.clone()],
             )),
             Nullability::Nullable,
         );
 
-        (f0_dt_null, f1_dt_null, dtype)
+        (f0_dt, f1_dt, dtype)
     }
 
     #[test]
+    #[should_panic]
     fn test_struct_scalar_null() {
-        let (f0_dt_null, f1_dt_null, dtype) = setup_types();
+        let (_, _, dtype) = setup_types();
 
         let scalar = Scalar::null(dtype);
 
-        let scalar_f0 = scalar.as_struct().field_by_idx(0);
-        assert!(scalar_f0.is_some());
-        let scalar_f0 = scalar_f0.unwrap();
-        assert_eq!(scalar_f0, Scalar::null(f0_dt_null.clone()));
-        assert_eq!(scalar_f0.dtype(), &f0_dt_null);
-
-        let scalar_f1 = scalar.as_struct().field_by_idx(1);
-        assert!(scalar_f1.is_some());
-        let scalar_f1 = scalar_f1.unwrap();
-        assert_eq!(scalar_f1, Scalar::null(f1_dt_null.clone()));
-        assert_eq!(scalar_f1.dtype(), &f1_dt_null);
+        scalar.as_struct().field_by_idx(0).unwrap();
     }
 
     #[test]
     fn test_struct_scalar_non_null() {
-        let (f0_dt_null, f1_dt_null, dtype) = setup_types();
+        let (f0_dt, f1_dt, dtype) = setup_types();
 
         let f0_val = Scalar::primitive::<i32>(1, Nullability::NonNullable);
         let f1_val = Scalar::utf8("hello", Nullability::NonNullable);
@@ -260,15 +244,15 @@ mod tests {
         let scalar = Scalar::struct_(dtype, vec![f0_val, f1_val]);
 
         let scalar_f0 = scalar.as_struct().field_by_idx(0);
-        assert!(scalar_f0.is_some());
+        assert!(scalar_f0.is_ok());
         let scalar_f0 = scalar_f0.unwrap();
         assert_eq!(scalar_f0, f0_val_null);
-        assert_eq!(scalar_f0.dtype(), &f0_dt_null);
+        assert_eq!(scalar_f0.dtype(), &f0_dt);
 
         let scalar_f1 = scalar.as_struct().field_by_idx(1);
-        assert!(scalar_f1.is_some());
+        assert!(scalar_f1.is_ok());
         let scalar_f1 = scalar_f1.unwrap();
         assert_eq!(scalar_f1, f1_val_null);
-        assert_eq!(scalar_f1.dtype(), &f1_dt_null);
+        assert_eq!(scalar_f1.dtype(), &f1_dt);
     }
 }
